const NodeChartOptions = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodeChartOptions.js');
const NodePurpose = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodePurpose.js');
window.ngui(() => {
    return {
        emits: [
            'update:modelValue',
            'submit:item',
            'remove:item',
            'update:purposes',
            'update:skipComputation'
        ],
        props: [
            'modelValue',
            'purposeOptions',
            'datapointListRules',
            'selectedAlgo',
            'maxItems',
            'hasExplanatoryVariables',
            'collectionLabel'
        ],        
        data() {
            return {
                localDatapointListRules: this.datapointListRules ?? [],
                defaultNode: {
                    unit: '',
                    interpolation: {
                        template: 'rawValues',
                    },                    
                    chartOptions: {
                        show: true,
                        y: {
                            ownAxis: true,
                            opposite: true,
                            min: undefined,
                            max: undefined,
                            title: '',
                        },
                        color: this.getRandomColorHex(),
                        stepline: false,
                        decimal: 0,
                        zIndex: 50,
                    },
                    extInfos: {
                        name: '',
                        unit: '',
                        accType: '',
                    },
                    purposes: [],  
                    bins: {
                        base: 0,
                        step: 3,
                    },               
                    pmModel: {
                        processManagerUuid: '',
                        path: '',
                        instanceUuid: '',
                        processUuid: '',
                    },                 
                },
            }
        },
        methods: { 
            getRandomColorHex() {
                return `#${Math.floor(Math.random() * 16777215).toString(16)}`;
            },
        },
        beforeMount() {
        },
        nodes() {
            return [              
                {
                    type: 'NgFormInput',
                    props: {
                        label: this.collectionLabel,
                        type: 'collection',                        
                        disableDrag: true,
                        maxItems: this.maxItems,
                        enableSelectAll: false,
                        modelValue: this.modelValue,
                        selectedAlgo: this.selectedAlgo,
                        hasExplanatoryVariables: this.hasExplanatoryVariables,
                        'onUpdate:modelValue': (value) => {
                            this.$emit('update:modelValue', value);
                        },                            
                        'onSubmit:item': (value) => {
                            this.$emit('submit:item', value);
                        }, 
                        'onRemove:item': (value) => {
                            this.$emit('remove:item', value);
                        },                                                                                                         
                        defaultValue: this.defaultNode,
                        validators: this.localDatapointListRules,
                    },
                    children: {
                        preside: () => {
                            return [
                                ''
                            ];
                        },                        
                        side: (slot) => {
                            return [
                                {
                                    type: 'NgMenu',
                                    props: {                                        
                                        icon: 'more_vert',
                                        flat: true,
                                        dense: true,
                                        round: true,                                 
                                        items: [
                                            {
                                                id: 'edit',
                                                label: 'edit',
                                                click: () =>  {
                                                    slot.editItem(slot.index);
                                                }
                                            },
                                            {
                                              id: 'delete',
                                              label: 'delete',
                                            //   disable: slot.index > 0 ? false : true,
                                              click: () => {
                                                    slot.dialog.delete = true;
                                                    slot.setSelectedIndex(slot.index);
                                                }
                                            }
                                        ]
                                    }                                    
                                }
                            ];
                        },
                        form: (slot) => {
                            const node = slot.items[slot.selectedIndex];
                            const vNodeChildren = [
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'datapoint',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                { 
                                                                    type: 'NgDMSPathChooser',
                                                                    props: {
                                                                        'data-cy': 'PathChooserInput',
                                                                        label: 'path',
                                                                        filter: [{ hasHistData: true }],
                                                                        required: true,
                                                                        // readonly: true,
                                                                        modelValue: node.pmModel,
                                                                        class: 'col-12',
                                                                        'onUpdate:modelValue': async (value) => {
                                                                            node.pmModel = value;
                                                                            const extInfos = await this.ctx.intelli.fetchExtInfos(value);  
                                                                            node.extInfos = extInfos;
                                                                            node.unit = '';
                                                                        },
                                                                        rules: [
                                                                            (val) => val !== '' || this.$t('ruleNotEmpty'),
                                                                            (val) => {
                                                                                const totalNodes = this.modelValue.filter((item) => { return item.pmModel.path === val })
                                                                                if (totalNodes.length != 0) {
                                                                                    const currentNode = this.modelValue[slot.selectedIndex];
                                                                                    if (currentNode && currentNode.pmModel.path == val)  {
                                                                                        return true;
                                                                                    }
                                                                                } else {
                                                                                    return true;
                                                                                }
                                                                                
                                                                                return this.$t('ruleDuplicateNode');
                                                                            },                                           
                                                                        ] 
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        label: 'unit',
                                                                        type: 'text',
                                                                        disable: !(node.extInfos && ('unit' in node.extInfos == false || node.extInfos.unit === '')),
                                                                        hint: this.$t('dmsUnit', {dmsUnit: node.extInfos?.unit || 'N/A'}),
                                                                        class: 'col-12',
                                                                        modelValue: node.unit,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            node.unit = value;
                                                                            this.$emit('update:skipComputation', true);
                                                                        },                                         
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: (this.purposeOptions && this.purposeOptions.length > 0) ? 'col-12' : 'hidden',
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'purpose',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                { 
                                                                    type: NodePurpose,
                                                                    props: {
                                                                        class: 'row col-12 q-col-gutter-sm',
                                                                        modelValue: node,
                                                                        selectedAlgo: this.selectedAlgo,
                                                                        purposeOptions: this.purposeOptions,
                                                                        'onUpdate:purposes': (value) => {                                        
                                                                            node.purposes = value;
                                                                            this.$emit('update:purposes', node);
                                                                        },
                                                                        'onUpdate:binBase': (value) => {
                                                                            node.bins.base = value;
                                                                        },
                                                                        'onUpdate:binStep': (value) => {
                                                                            node.bins.step = value;
                                                                        },                                                                                                                                                                                         
                                                                    },
                                                                },                                                               
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                { 
                                    type: 'NgFormGroups',
                                    props: {
                                        class: 'col-12',
                                    },
                                    children: {
                                        default: (slot) => {
                                            return [
                                                { 
                                                    type: 'NgFormGroupItem',
                                                    props: {
                                                        title: "Interpolation",
                                                    },
                                                    children : {
                                                        default: () => {
                                                            return [
                                                                { 
                                                                    type: 'NgBIInterpolation',
                                                                    props: {
                                                                        modelValue: node.interpolation,
                                                                        class: 'row col-12 q-col-gutter-sm',
                                                                        'onUpdate:modelValue': (value) => {
                                                                            node.interpolation = value;
                                                                        },
                                                                    },
                                                                },
                                                            ];
                                                        }
                                                    }
                                                },  
                                            ];
                                        }
                                    }
                                },                                  
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'options',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                { 
                                                                    type: NodeChartOptions,
                                                                    props: {
                                                                        class: 'q-col-gutter-sm row',
                                                                        modelValue: node,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            node = value;
                                                                        },
                                                                        'onUpdate:skipComputation': (value) => {
                                                                            this.$emit('update:skipComputation', value);
                                                                        },                                                                        
                                                                    },        
                                                                },                                                               
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                                                                                                                                                      
                            ];
                            return vNodeChildren;
                            
                        },
                        item: (slot) => {
                            const node = slot.item;
                            const pmModel = node.pmModel;
                            const name = node.extInfos.name || '';
                            const unit = node.extInfos.unit || node.unit || '-';

                            let purposes = '';
                            if ('purposes' in node) {
                                purposes = '/ ' + node.purposes.map((purpose) => this.$t(purpose)).join(', ');
                            }

                            const nodeName = [name, ' ['+ unit +'] ', pmModel.path + ' /', this.ctx.intelli.getInterpolationName(node.interpolation), purposes];
                            return [
                                { 
                                    type: 'NgText',
                                    props: {
                                        text:  nodeName,
                                        // weight: (pmModel.path === this.mainNode.pmModel.path) ? 'bold' : ''
                                    },                                
                                }, 
                            ]
                        },                                                                                                                         
                    },
                },
            ]
        },
    }
  });