window.ngui(() => {
    return {
        emits: [
            'update:modelValue'
        ],
        props: [
            'modelValue',
            'renderIgnoreEvents',
            'trainingPeriodRules'
        ],
        data() {
            return {
                localModelValue: {},
                localTrainingPeriodRules: [],
            }
        },
        methods: { 
            getAmountDaysBetweenTwoDates(startDate, endDate) {
                const start = new Date(startDate);
                const end = new Date(endDate);
                const diffInTime = end.getTime() - start.getTime();
                const diffInDays = diffInTime / (1000 * 3600 * 24);
                return diffInDays;        
            }
        },
        beforeMount() {
            this.localModelValue = this.modelValue;
            this.localTrainingPeriodRules = this.trainingPeriodRules || [];
        },
        nodes() {
            const vNodes =  [                 
                {
                    type: 'NgFormInput',
                    props: {
                        col: 6,
                        class: `q-pr-sm`,
                        label: 'trainingPeriodDay',
                        type: 'number',
                        modelValue: this.localModelValue.periodsDay.train,
                        'onUpdate:modelValue': (value) => {
                            this.localModelValue.periodsDay.train = value;
                            this.$emit('update:modelValue', this.localModelValue);
                        },
                        rules: [
                            (val) => val !== '' && val !== undefined|| this.$t('ruleNotEmpty'),
                            () => {
                                let rv = true;
                                if (this.localModelValue.trainMethod == 'movingWindows' && this.localModelValue.trainingDates.length > 0) {     
                                    const diffInDays = this.getAmountDaysBetweenTwoDates(
                                        this.localModelValue.trainingDates[0].start, 
                                        this.localModelValue.trainingDates[0].end
                                    );
                                    const periodsDaySum = this.localModelValue.periodsDay.train + this.localModelValue.periodsDay.retrain;
                                    if (diffInDays < periodsDaySum) {
                                        rv = this.$t('ruleDateSpanSmallerThanTrainPeriods', { diffInDays, periodsDaySum }); 
                                        
                                    }  
                                }
                                return rv;
                            },
                        ].concat(this.localTrainingPeriodRules)
                    },
                },   
                {
                    type: 'NgFormInput',
                    props: {
                        col: 6,
                        label: 'retrainingPeriodDay',
                        type: 'number',
                        modelValue: this.localModelValue.periodsDay.retrain,
                        'onUpdate:modelValue': (value) => {
                            this.localModelValue.periodsDay.retrain = value;
                            this.$emit('update:modelValue', this.localModelValue);
                        },
                        rules: [
                            (val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'),
                            (val) => this.localModelValue.interval !== -2 || this.localModelValue.interval === -2 && val >= 7 || this.$t('ruleMinDays', {total: 7})
                        ]
                    },
                },                         
            ];


            if (this.renderIgnoreEvents) {
                vNodes.push(
                    {
                        type: 'NgFormInput',
                        props: {
                            col: 12,
                            label: 'ignoreEvents',
                            type: 'checkbox',
                            modelValue: this.localModelValue.periodsDay.ignoreEvents,
                            'onUpdate:modelValue': (value) => {
                                this.localModelValue.periodsDay.ignoreEvents = value;
                                this.$emit('update:modelValue', this.localModelValue);
                            },                                         
                        },
                    },  
                )
            }
            
            return vNodes;
        },
    }
  });