const NodeCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodeCollection.js');
const TrainingDatesCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainingDateCollection.js');
const TrainDateSelector = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainDateSelector.js');
const PeriodsDay = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/periodsDay.js');

window.ngui(() => {
    return {        
        emits: [
            'form:submit',
        ],
        props: [
            'payload', 
        ],
        methods: {
            setDependentNode(node) {
                this.dependentNode = node;
                this.dependentNodeCollection = [node];
            },
            regionOptionsHasCountry() {
                return Object.keys(this.holidayRegionOptions).includes(this.configModel.holidayLocation.country);
            },
            getMonthDelta(interval) {
                return interval < 0 ? 36 : 3;
            },
            togglePeriodsDayVisibility(trainMethod) {
                this.periodsDayClass = trainMethod === 'movingWindows' ? '' : 'hidden';
            },
            resetTrainingDates() {
                this.configModel.trainingDates = [];
                this.localTrainDates = [];
            },
            hasExplanatoryVariables() {
                let hasExplanatoryVariables = false
                if (this.configModel.explanatoryDateTimeVariables) {
                    if (this.configModel.explanatoryDateTimeVariables.length > 0) {
                        hasExplanatoryVariables = true
                    }
                }

                const explVars = this.independentNodes.filter((node) => {
                    if ('purposes' in node) {
                        return node.purposes.includes('computeThresholds') || false;
                    }
                    return false;
                });
                if (explVars.length > 0) {
                    hasExplanatoryVariables = true;
                }

                if (this.configModel.holidayLocation) {
                    if (this.configModel.holidayLocation.country !== '') {
                        hasExplanatoryVariables = true
                    }
                }                
                return hasExplanatoryVariables;
            }           
        },
        // The returned object can be accessed with this within any method...
        // Do not use this.ctx within this function because it is not yet created.
        data() {
            return {
                algorithm: '',
                configModelLoaded: false,
                intervalOptions:  [
                    {label: 'hourly', value: 3600},
                    {label: 'daily', value: -1},
                    // {label: 'weekly', value: -2},
                ],
                dependentNodeCollection: [], // interface var for collection
                dependentNode: {},
                independentNodes: [],
                localTrainMethod: 'movingWindows',
                localTrainDates: [],
                periodsDayClass: '',
                explanatoryDateTimeOptions: [
                    {label: 'hod', value: 'hod'},
                    {label: 'dow', value: 'dow'},
                    {label: 'woy', value: 'woy'},
                    {label: 'month', value: 'month'},                    
                    {label: 'quarter', value: 'quarter'},                    
                    {label: 'Mo2Sa-So', value: 'Mo2Sa-So'},
                    {label: 'Mo2Fr-Sa-So', value: 'Mo2Fr-Sa-So'},        
                    {label: 'Mo2Fr-Sa2So', value: 'Mo2Fr-Sa2So'},
                ],
                holidayCountryOptions: [
                    {label: '', value: ''},
                    {label: 'switzerland', value: 'CH'},
                    {label: 'germany', value: 'DE'},
                    {label: 'luxembourg', value: 'LU'},
                    {label: 'austria', value: 'AT'},
                    {label: 'netherlands', value: 'NL'},
                ],
                holidayRegionOptions: {
                    'default': [
                        {label: '', value: ''},
                    ],
                    'CH': [
                        {label: '', value: ''},
                        {label: 'ZH', value: 'ZH'},
                        {label: 'BE', value: 'BE'},
                        {label: 'LU', value: 'LU'},
                        {label: 'UR', value: 'UR'},
                        {label: 'SZ', value: 'SZ'},
                        {label: 'OW', value: 'OW'},
                        {label: 'NW', value: 'NW'},
                        {label: 'GL', value: 'GL'},
                        {label: 'ZG', value: 'ZG'},
                        {label: 'FR', value: 'FR'},
                        {label: 'SO', value: 'SO'},
                        {label: 'BS', value: 'BS'},
                        {label: 'BL', value: 'BL'},
                        {label: 'SH', value: 'SH'},
                        {label: 'AR', value: 'AR'},
                        {label: 'AI', value: 'AI'},
                        {label: 'SG', value: 'SG'},
                        {label: 'GR', value: 'GR'},
                        {label: 'AG', value: 'AG'},
                        {label: 'TG', value: 'TG'},
                        {label: 'VD', value: 'VD'},
                        {label: 'TI', value: 'TI'},
                        {label: 'VS', value: 'VS'},
                        {label: 'NE', value: 'NE'},
                        {label: 'GE', value: 'GE'},
                        {label: 'JU', value: 'JU'},
                    ],
                    'DE': [
                        {label: '', value: ''},
                        {label: 'BW', value: 'BW'},
                        {label: 'BY', value: 'BY'},
                        {label: 'BE', value: 'BE'},
                        {label: 'BB', value: 'BB'},
                        {label: 'HB', value: 'HB'},
                        {label: 'HH', value: 'HH'},
                        {label: 'HE', value: 'HE'},
                        {label: 'MV', value: 'MV'},
                        {label: 'NI', value: 'NI'},
                        {label: 'NW', value: 'NW'},
                        {label: 'RP', value: 'RP'},
                        {label: 'SL', value: 'SL'},
                        {label: 'SN', value: 'SN'},
                        {label: 'ST', value: 'ST'},
                        {label: 'SH', value: 'SH'},
                        {label: 'TH', value: 'TH'},
                    ]               
                },
                dialog: {
                    chart: false,
                },
                trainMethodOptions: [
                    {label: 'movingWindows', value: 'movingWindows'},
                    {label: 'isolatedWindows', value: 'isolatedWindows'}                
                ],
                configModel: {
                    uuid: '',
                    task: {
                        name: 'maxpermissiblevalues',
                        algorithm: {
                            method: '',
                            model: '',
                        },
                        metric: undefined,
                    }, 
                    interval: -1,
                    nodes: [],
                    dependentVariables: [],
                    independentVariables: [],
                    explanatoryDateTimeVariables: [],
                    holidayLocation: {
                        country: '',
                        region: '',
                    },
                    minElementsPerBin: 3,
                    trainMethod: 'movingWindows',
                    trainTime: '00:00', 
                    trainingDates: [],
                    measureDates: [],
                    periodsDay: {
                        train: 60,
                        retrain: 20,
                        ignoreEvents: false,
                    },
                    sensitivityThreshold: {
                        upper: 1,
                        lower: 1,
                    },
                    eventCount: {
                        upper: 3,
                        lower: 3,
                    },
                    minErrorDistance: {
                        upper: 0,
                        lower: 0,
                    },
                    trespassing: {
                        upper: true,
                        lower: true,                        
                    },
                    thresholdColor: '#99ff99',
                    expectedValueColor: '#47b547',
                },                        
                modelOptions: [                                      
                    '',
                    'groupingMethod', // Grouping method
                    'AdaBoostRegressor',
                    'BaggingRegressor',
                    'DecisionTreeRegressor',
                    'ExtraTreesRegressor',
                    'GammaRegressor',
                    'GaussianProcessRegressor',
                    'GradientBoostingRegressor',
                    'HistGradientBoostingRegressor',
                    'HuberRegressor',
                    'KNeighborsRegressor',
                    'KernelRidge',
                    'Lars',
                    'LarsCV',
                    'LassoCV',
                    'LassoLarsCV',
                    'LassoLarsIC',
                    'LinearRegression',
                    'LinearSVR',
                    'MLPRegressor',
                    'NuSVR',
                    'OrthogonalMatchingPursuit',
                    'OrthogonalMatchingPursuitCV',
                    'PiecewiseRegressor',                    
                    'RANSACRegressor',
                    'RadiusNeighborsRegressor',
                    'RandomForestRegressor',
                    'RidgeCV',
                    'SVR',
                    'TheilSenRegressor',
                    'XGBoost',
                ] 
            }       
        },
        beforeMount() {
            this.widget = this.payload.widget;
            const now = this.ctx.moment.moment();
            const defaultTrainingDates = {
                start: '',
                end: now.set({minute:0, second:0, millisecond:0}).format()
            }
            const months = this.getMonthDelta(this.configModel.interval);
            defaultTrainingDates.start = now.subtract(months, 'months').format();

            this.configModel.trainingDates.push(defaultTrainingDates);
            this.dependentNode = this.ctx.intelli.initNode();
            this.dependentNode.uuid = this.ctx.uuidv4();
                        
            // Get config model
            this.ctx.filedb.read('app/BAI/configModel/'+ this.widget.configModelUuid + '.json', true)
            .then((response) => {
                const configModel = response.data.data;
                this.configModel = {...this.configModel, ...configModel};

                const dependentNodes = this.configModel.nodes.filter((node) => {
                    return (this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });
                if (dependentNodes.length > 0) {
                    this.setDependentNode(dependentNodes[0]);                    
                }                

                this.independentNodes = this.configModel.nodes.filter((node) => {
                    // Filter all remaining nodes except dependent node
                    // Use dependentVarialbes since independentVariables contains only the ones with enabled purpose
                    return !(this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });

                this.localTrainMethod = this.configModel.trainMethod;
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                

                this.algorithm = this.configModel.task.algorithm.model;

                this.configModelLoaded = true;
            })
            .catch(() => {
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                    
            });                
        },
        // Define elements within the ui configuration...
        nodes() {
            return [
                {
                    type: 'NgDialog',
                    props: {
                        title: '',
                        maximizing: true,
                        modelValue: this.dialog.chart,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.chart = value;
                        },                        
                        width: '1000px',
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                        onHide: () => {
                            this.configModel.trainMethod = this.localTrainMethod;
                            this.configModel.trainingDates = this.localTrainDates;
                        },
                    },
                    children: {
                        default: (slot) => {                            
                            return [   
                                {
                                    type: TrainDateSelector,
                                    props: {
                                        trainMethodOptions: this.trainMethodOptions,
                                        trainMethod: this.configModel.trainMethod,
                                        trainDates: this.configModel.trainingDates,
                                        nodeCollection: this.configModel.nodes,
                                        interval: this.configModel.interval,
                                        intervalOptions: this.intervalOptions,                        
                                        'onUpdate:interval': (value) => {
                                            this.configModel.interval = value;
                                        },                                        
                                        'onUpdate:nodeCollection': (values) => {
                                            this.configModel.nodes = values;
                                        },
                                        'onUpdate:trainDates': (trainDates) => {
                                            this.localTrainDates = trainDates;                                                   
                                        },                                   
                                        'onUpdate:trainMethod': (value) => {
                                            this.localTrainMethod = value;
                                            this.togglePeriodsDayVisibility(value);
                                        },
                                        'onSubmit': () => {
                                            this.dialog.chart = false;
                                        }
                                    }
                                }
                            ];
                        }
                    }
                },                        
                {
                    type: 'NgForm',                
                    props: {
                        greedy: true,
                        submitLabel: 'computeThresholds',
                        'data-cy': 'computeThresholdsForm',
                        key: this.configModelLoaded,
                        onSubmit: async () =>  {
                            try {
                                this.configModel.nodes = [...this.independentNodes, this.dependentNode];

                                // (#54012) Reset measure dates for rollingWindows
                                this.configModel.measureDates = [];

                                // Set transform dates from latest end date to today
                                if (this.configModel.trainMethod === 'isolatedWindows') {
                                    const trainingEndDates = this.configModel.trainingDates.map((date) => {
                                        return date.end;
                                    })
                                    const today = this.ctx.moment.moment().format();
                                    const lastEndDate = trainingEndDates.sort().reverse()[0];
                                    this.configModel.measureDates = [{
                                        'start': lastEndDate,
                                        'end': today
                                    }];
                                }

                                this.configModel.dependentVariables = [`${this.dependentNode['pmModel']['processUuid']}_${this.dependentNode['pmModel']['path']}`];

                                // Update independentVariables
                                this.configModel.independentVariables = [];                    
                                this.independentNodes.forEach((node) => {
                                    if ('purposes' in node && node['purposes'].includes('computeThresholds')) {
                                        const nodeIdentifier = `${node['pmModel']['processUuid']}_${node['pmModel']['path']}`
                                        this.configModel.independentVariables.push(nodeIdentifier);
                                    }
                                })
                                this.configModel.independentVariables = this.configModel.independentVariables.concat(this.configModel.explanatoryDateTimeVariables);
                                    
                                if (this.configModel.holidayLocation) {
                                    if (this.configModel.holidayLocation.country !== '') {
                                        this.configModel.independentVariables.push('holiday');
                                    }
                                }

                                // set uuids
                                this.configModel.nodes = this.configModel.nodes.map((node) => {
                                    if (!node.uuid) {
                                        node.uuid = this.ctx.uuidv4();
                                    }
                                    return node;
                                });

                                await this.ctx.filedb.insert('app/BAI/configModel/'+ this.configModel.uuid + '.json', this.configModel);                                

                                const nodes = this.configModel.nodes?.map((node) => {
                                    const dependentVariableNames = this.configModel.dependentVariables;
                                    return {
                                        uuid: node.uuid,                               
                                        dependentVariable: (dependentVariableNames.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`)),
                                        interpolation: node.interpolation,
                                        bins: node.bins,
                                        pmModel: node.pmModel,
                                        extInfos: node.extInfos,
                                        chartOptions: node.chartOptions,
                                    };
                                  });

                                  const nodeOptions = this.configModel.nodes?.map((node) => {
                                    return {
                                      name: this.ctx.intelli.getNodeChartName(node),
                                      type: 'line',
                                      data: {
                                        yUuidRef: node.uuid,
                                      },
                                      yAxis: this.ctx.intelli.getNodeChartName(node),
                                    }
                                  });

                                const trainingDates = this.configModel.trainingDates.map((dates) => {
                                    let end = dates.end;
                                    if (!dates.end) {
                                        end = this.ctx.moment.getLocalizedNow().format()
                                    }
                                    return {
                                        start: dates.start,
                                        end
                                    }
                                });

                                const dependentNode = this.configModel.nodes?.filter((node) => this.configModel.dependentVariables?.includes(`${node['pmModel']['processUuid']}_${node['pmModel']['path']}`))[0];
                                const payload = {
                                    scriptType: 'maxpermissiblevalues',
                                    scriptFile: 'compute_thresholds.py',
                                    scriptArguments: {
                                        configModelUuid: this.configModel.uuid,
                                        trainingDates: trainingDates,
                                        measureDates: this.configModel.measureDates,
                                        dependentNodeName: this.ctx.intelli.getNodeChartName(dependentNode)
                                    },
                                    dates: trainingDates.concat(this.configModel.measureDates),
                                    nodes,
                                    nodeOptions,
                                    holidayLocation: this.configModel.holidayLocation,
                                    histData: {
                                        start: '',
                                        end: '',
                                        interval: this.configModel.interval,
                                    }
                                }

                                if (this.widget.name === '') {
                                    this.widget.name = this.dependentNode['pmModel']['path'];
                                    if (this.dependentNode['extInfos']['name'] !== '') {
                                        this.widget.name = `${this.dependentNode['extInfos']['name']} - ${this.dependentNode['pmModel']['path']}`;
                                    }
                                }
                                await this.ctx.filedb.insert('app/BAI/widgets/'+ this.widget.uuid + '.yaml', this.widget);                                

                                this.$emit('form:submit', payload);
                            } catch (err) {
                                this.ctx.log.error(err);
                            }
                        },
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'observedDatapoint',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: NodeCollection,
                                                                    props: {
                                                                        modelValue: this.dependentNodeCollection,
                                                                        purposeOptions: [],
                                                                        selectedAlgo: this.configModel.task.algorithm,
                                                                        maxItems: 1,
                                                                        'onUpdate:modelValue': (values) => {                                                                            
                                                                            this.dependentNodeCollection = [];
                                                                            if (values.length > 0) {
                                                                                this.setDependentNode(values[0]);
                                                                            }
                                                                            // keep this.configModel.nodes update to date for trainDateSelector
                                                                            this.configModel.nodes = [values[0], ...this.independentNodes];
                                                                        },                                                                          
                                                                        datapointListRules: [
                                                                            `
                                                                            (function() {
                                                                                let rv = $t('ruleMinElement', {'total': 1});
                                                                                if (props.modelValue.length > 0) {
                                                                                    rv = true;
                                                                                }
                                                                                return rv;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),     
                                                                        ]
                                                                    },
                                                                },                                                           
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                        separator: false,
                                        bordered: false,
                                        dense: true,

                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeResolution',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'interval',
                                                                        type: 'select',
                                                                        options: this.intervalOptions,
                                                                        modelValue: this.configModel.interval,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.interval = value;
                                                                            if (this.configModel.trainingDates.length > 0) {
                                                                                const months = this.getMonthDelta(this.configModel.interval);
                                                                                // Set default training dates on time resolution update.
                                                                                if (this.configModel.trainMethod === 'movingWindows') {
                                                                                    let endDate = this.ctx.moment.getLocalizedNow();
                                                                                    if (this.configModel.trainingDates[0].end !== '') {
                                                                                        endDate = this.ctx.moment.getDate(this.configModel.trainingDates[0].end)
                                                                                    }
                                                                                    this.configModel.trainingDates[0].start = endDate.subtract(months, 'months').format();                                            
                                                                                }
                                                                            }
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty')]                                                                             
                                                                    },
                                                                },                                                                   
                                                            ]
                                                        }
                                                    }
                                                }, 
                                            ]
                                        },
                                    }
                                },                                         
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'algorithm',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'model',                                        
                                                                        type: 'select',
                                                                        options: this.modelOptions.map((option) => { 
                                                                            return { label: option, value: option };
                                                                        }),
                                                                        modelValue: this.algorithm,                                                                          
                                                                        'onUpdate:modelValue': (value) => {      
                                                                            this.algorithm = value;                                                                  
                                                                            this.configModel.task.algorithm.method = 'thresholdsRegression';                                           
                                                                            this.configModel.task.algorithm.model = value;

                                                                            if (value === 'groupingMethod') {
                                                                                this.configModel.task.algorithm.method = 'thresholdsGrouping';
                                                                                this.configModel.task.algorithm.model = '';
                                                                            }
                                                                        },
                                                                        rules: [
                                                                            (val) => ((val !== undefined && val !== '')) || this.$t('ruleNotEmpty')
                                                                        ]                                                                                                      
                                                                    },
                                                                },                                                                 
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                                                                                                                                                                                                                                                                                                                                                                              
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [                                                    
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'explanatoryVariables',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: NodeCollection,
                                                                    props: {
                                                                        modelValue: this.independentNodes,
                                                                        purposeOptions: ['computeThresholds'],
                                                                        collectionLabel: 'datapointList',
                                                                        selectedAlgo: this.configModel.task.algorithm,
                                                                        hasExplanatoryVariables: this.hasExplanatoryVariables(),
                                                                        'onUpdate:modelValue': (values) => {
                                                                            this.independentNodes = values;
                                                                            // keep this.configModel.nodes update to date for trainDateSelector
                                                                            this.configModel.nodes = [this.dependentNode, ...values];
                                                                        },                                                                          
                                                                        datapointListRules: [
                                                                            `
                                                                            (function() {
                                                                                if (!attrs.hasExplanatoryVariables) {
                                                                                    return $t('ruleAtLeastOneExplanatoryVariableWithPurpose');
                                                                                }                                                                                
                                                                                return true;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),                                                                             
                                                                            `
                                                                            (function() {
                                                                                if (attrs.selectedAlgo.method === 'thresholdsGrouping') {
                                                                                    const errors = val.filter((node) => {
                                                                                        if (node['extInfos']['accType'] === 'double64') {
                                                                                            if ('bins' in node) {
                                                                                                if ('base' in node['bins'] === false || 'step' in node['bins'] === false) {
                                                                                                    return true;
                                                                                                }
                                                                                                if ([node['bins']['base'], node['bins']['step']].includes(undefined)) {
                                                                                                    return true;
                                                                                                }
                                                                                            }
                                                                                        }                                                                                     
                                                                                    });     
                                                                                    if (errors.length > 0) {
                                                                                        return $t('ruleAlgoNeedsBinning');
                                                                                    }
                                                                                }
                                                                                return true;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),                                                                                                                                             
                                                                        ]
                                                                    },
                                                                },                                                                
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'dateTimeVariables',
                                                                        type: 'select',
                                                                        multiple: true,
                                                                        options: this.explanatoryDateTimeOptions,
                                                                        modelValue: this.configModel.explanatoryDateTimeVariables,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.explanatoryDateTimeVariables = value;
                                                                        },
                                                                        rules: [
                                                                            () => this.hasExplanatoryVariables() || this.$t("ruleAtLeastOneExplanatoryVariableWithPurpose")
                                                                        ]                                                                       
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'holidaysCountry',
                                                                        type: 'select',
                                                                        options: this.holidayCountryOptions,
                                                                        modelValue: this.configModel.holidayLocation.country,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.holidayLocation.country = value;
                                                                            this.configModel.holidayLocation.region = '';
                                                                        }                                                                    
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'holidaysRegion',
                                                                        disable: !this.regionOptionsHasCountry(),
                                                                        type: 'select',
                                                                        options: this.holidayRegionOptions[this.regionOptionsHasCountry() ? this.configModel.holidayLocation.country : 'default'],
                                                                        modelValue: this.configModel.holidayLocation.region,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.holidayLocation.region = value;
                                                                        },
                                                                        rules: [
                                                                            (val) => {
                                                                                if (this.regionOptionsHasCountry()) {
                                                                                    return val ? true : this.$t('ruleNotEmpty');
                                                                                }
                                                                                return true;
                                                                            }
                                                                        ]                                                                                                                                         
                                                                    },
                                                                },                                                                                                                        
                                                                // {
                                                                //     type: 'NgFormInput',
                                                                //     props: {
                                                                //         col: 6,
                                                                //         label: 'minElementsPerBin',
                                                                //         type: 'number',
                                                                //         modelValue: this.configModel.minElementsPerBin,
                                                                //         'onUpdate:modelValue': (value) => {
                                                                //             this.configModel.minElementsPerBin = value;
                                                                //         },
                                                                //         rules: [(val) => val !== "" || this.$t("ruleNotEmpty")]                                                                          
                                                                //     },
                                                                // },                                                                                                                               
                                                            ];
                                                        }
                                                    }
                                                },  
                                            ]
                                        }
                                    }                                    
                                },
                                {
                                    type: 'NgFormGroups',                                
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeSpans',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'trainMethod',
                                                                        type: 'select',
                                                                        modelValue: this.configModel.trainMethod,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainMethod = value;
                                                                            this.localTrainMethod = value;
                                                                            this.togglePeriodsDayVisibility(value);
                                                                            this.resetTrainingDates();
                                                                        },
                                                                        options: this.trainMethodOptions,
                                                                        rules: [
                                                                            (val) => val !== '' || this.$t('ruleNotEmpty'),
                                                                            () => this.configModel.trainingDates.length > 0 || this.$t('ruleMinElement', {total: 1}),
                                                                        ]
                                                                    },
                                                                },                                                             
                                                                {
                                                                    type: 'div',
                                                                    props: {
                                                                        class: 'row col-12',
                                                                    },
                                                                    children: {
                                                                        default: (slot) => {                            
                                                                            return [
                                                                                {
                                                                                    type: 'NgActionButton',
                                                                                    props: {
                                                                                        fullWidth: true,
                                                                                        color: this.dependentNodeCollection.length > 0 ? 'primary' : 'grey',
                                                                                        disable: this.dependentNodeCollection.length > 0 ? false : true,
                                                                                        label: 'visualizeDatapoint',
                                                                                        onClick: () => {
                                                                                            this.dialog.chart = true;
                                                                                        }    
                                                                                    },
                                                                                },                                                                                    
                                                                            ]
                                                                        }
                                                                    }
                                                                },                                                              
                                                                {
                                                                    type: TrainingDatesCollection,
                                                                    props: {
                                                                        class: 'col-12',
                                                                        trainMethod: this.configModel.trainMethod,
                                                                        modelValue: this.configModel.trainingDates,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainingDates = value;
                                                                        },                                        
                                                                    },
                                                                },                                                            
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                 
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12 ' + this.periodsDayClass                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'trainingPeriod',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            const trainingPeriodRules = [];
                                                            if (this.configModel.task.algorithm.method === 'thresholdsGrouping') {
                                                                trainingPeriodRules.push(
                                                                    () => {
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('quarter')
                                                                            && this.configModel.periodsDay.train <= 90) {
                                                                                return 'thresholdsGroupingAndQuarterAtLeast90Days';
                                                                        }
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('month')
                                                                            && this.configModel.periodsDay.train <= 30) {
                                                                                return 'thresholdsGroupingAndMonthAtLeast30Days';
                                                                        }                                    
                                                                        return true;
                                                                    });
                                                            }
                                                            return [
                                                                {
                                                                    type: PeriodsDay,
                                                                    props: {
                                                                        class: 'row',
                                                                        modelValue: this.configModel,
                                                                        renderIgnoreEvents: false,
                                                                        trainingPeriodRules,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel = value;
                                                                        }                                                                     
                                                                    },
                                                                },                                                                
                                                            ];
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'sensitivity',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'upper',
                                                                        type: 'slider',
                                                                        modelValue: this.configModel.sensitivityThreshold.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.upper = value;
                                                                        },
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        reverse: true,
                                                                        required: true,
                                                                        default: 1,                          
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'lower',
                                                                        type: 'slider',
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        required: true,
                                                                        default: 1,                                   
                                                                        modelValue: this.configModel.sensitivityThreshold.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.lower = value;
                                                                        },
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'eventCount',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.upper = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.lower = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined  || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'minErrorDistance',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.upper = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.lower = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 ||this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'thresholdTrespassingActivation',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'toggle',
                                                                        modelValue: this.configModel.trespassing.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trespassing.upper = value;
                                                                        },
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'toggle',
                                                                        modelValue: this.configModel.trespassing.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trespassing.lower = value;
                                                                        },
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                             
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'visualization',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'thresholdColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.thresholdColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.thresholdColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'expectedValueColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.expectedValueColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.expectedValueColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },                                                                                                                                                                              
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                }                                                                                               
                            ]                                            
                        },
                    },                   
                }
            ]
        },
    }
});