from utils import get_model_params
import json
import sys, os
currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)
from intelli_utilities import fetch_config_model, fetch_outputs, LOCAL_TIME_ZONE
from datetime import datetime
from lib.Thresholds import Thresholds
import pandas as pd

def main(args):
    custom_payload = args['customPayload']
    config_model_uuid = custom_payload['configModelUuid']
    dependent_node_visualization_name = custom_payload['dependentNodeName']    

    input_uuid = args['inputDataUuid']
    output_uuid = args['outputUuid']
    token = args['token']
    process_manager = args['processManager']    
    
    outputs = fetch_outputs(output_uuid, token, process_manager)['data']
    thresholds_df = pd.DataFrame(outputs['thresholdsDataFrame'])
    thresholds_df = thresholds_df.set_index('dateTime', drop=False)
    thresholds_df.index = pd.to_datetime(thresholds_df.index)    
    
    config_model = fetch_config_model(config_model_uuid, token, process_manager)    
    dependent_node_df_colname = config_model['dependentVariables'][0]

    new_rows = pd.DataFrame()
    
    thresholds = Thresholds()
    thresholds.parse_config_model(config_model)

    # Load input data
    with open('spool/{}/input.json'.format(input_uuid), 'r') as file:
        input_data = json.load(file)

    dataFrame = pd.DataFrame(input_data['dataFrame'])
    dataFrame = dataFrame.set_index('dateTime', drop=False)
    dataFrame.index = pd.to_datetime(dataFrame.index)             

    train_df = pd.DataFrame()
    if len(custom_payload['trainingDates']) > 0:
        dataframes = []
        for date in custom_payload['trainingDates']:
            dt_start = datetime.fromisoformat(date['start']).astimezone(LOCAL_TIME_ZONE)
            dt_end = datetime.fromisoformat(date['end']).astimezone(LOCAL_TIME_ZONE)
            dataframes.append(dataFrame[dt_start:dt_end])
        train_df = pd.concat(dataframes)

    transform_df = pd.DataFrame()
    if len(custom_payload['measureDates']) > 0:
        dataframes = []
        for date in custom_payload['measureDates']:
            dt_start = datetime.fromisoformat(date['start']).astimezone(LOCAL_TIME_ZONE)
            dt_end = datetime.fromisoformat(date['end']).astimezone(LOCAL_TIME_ZONE)  
            dataframes.append(dataFrame[dt_start:dt_end])
        transform_df = pd.concat(dataframes)


    if config_model['trainMethod'] == 'movingWindows':
        # Synchronize time on weekly interval
        # if config_model['interval'] == INTERVAL['week']:                
        #     i = thresholds_df.index.searchsorted(training_start_date.astimezone(LOCAL_TIME_ZONE))
        #     row = thresholds_df.iloc[[i]]
        #     training_start_date = row.index[0]
                  

        model_params = get_model_params(config_model)
        threshold_instance = thresholds.create_thresold_instance(config_model['task']['algorithm']['method'], config_model['dependentVariables'], config_model['independentVariables'], config_model['task']['algorithm']['model'], model_params)  
        updated_outputs = thresholds.do_backtesting(train_df, threshold_instance)
        updated_threshold_df = updated_outputs['thresholds']

        # Append only new rows since we retrain the current transformed date span                    
        list_indexes = list(thresholds_df.index)
        new_rows = updated_threshold_df[~updated_threshold_df.index.isin(list_indexes)]
                                                                        
    if config_model['trainMethod'] == 'isolatedWindows':
        if transform_df.empty == False: # having test data to a limit date results in empty dataframe when updating the last data points
            model_params = get_model_params(config_model)
            threshold_instance = thresholds.create_thresold_instance(config_model['task']['algorithm']['method'], config_model['dependentVariables'], config_model['independentVariables'], config_model['task']['algorithm']['model'], model_params)
            outputs, _ = thresholds.do_single_train(threshold_instance, train_df, transform_df)                
            new_rows = threshold_instance.transform(transform_df)
                            
    thresholds_df = pd.concat([thresholds_df, new_rows])                                
    thresholds_df = thresholds.set_event_flags(thresholds_df)                                            
    thresholds_df = thresholds.set_alarm_flags(thresholds_df, True)
    thresholds_df = thresholds.bake_thresholds(thresholds_df)          

    events_df = thresholds_df[(thresholds_df['event_upper'] == True) | (thresholds_df['event_lower'] == True)]
    alarms_df = thresholds_df[(thresholds_df['alarm_upper'] == True) | (thresholds_df['alarm_lower'] == True)]
        
    response = {
        'line': [
            { 
                'id': 'expectedValue',
                'name': 'expectedValue',
                'x': thresholds_df['dateTime'].values.tolist(),
                'y': thresholds_df['Y_pred'].values.tolist(),
                'color': config_model['expectedValueColor'],
                'yaxis': dependent_node_visualization_name,
                'zIndex': 50       
            }
        ],
        'rangeArea': [
            {
                'id': 'thresholds',                
                'name': 'thresholds',
                'x': thresholds_df['ts'].values.tolist() ,
                'y': thresholds_df['lower'].values.tolist(),
                'y2': thresholds_df['upper'].values.tolist(),
                'color': config_model['thresholdColor'],
                'yaxis': dependent_node_visualization_name,
                'zIndex': 10
            }
        ], 
        'annotations': {
            'points': [
                {
                    'x': events_df['dateTime'].values.tolist(),
                    'y': events_df[dependent_node_df_colname].values.tolist(),
                    'color': 'blue',
                    'shape': 'circle',
                    'radius': 4,
                    'size': 5,
                    'yaxis': dependent_node_visualization_name
                },
                {
                    'x': alarms_df['dateTime'].values.tolist(),
                    'y': alarms_df[dependent_node_df_colname].values.tolist(),
                    'color': 'red',
                    'shape': 'circle',
                    'radius': 4,
                    'size': 5,
                    'yaxis': dependent_node_visualization_name
                }                
            ]
        },
        'thresholdsDataFrame' : thresholds_df.to_dict('records')
    }
    
    if config_model['trainMethod'] == 'movingWindows':
        response['trainingDates'] = outputs['trainingDates'] + updated_outputs['trainingDates']
        response['reTrainingDates'] = outputs['reTrainingDates'] + updated_outputs['reTrainingDates']
    
    return response      
        
if __name__ == "__main__":       
    response = main(runpy_args)