import socket
import argparse
import runpy
import json
from http import HTTPStatus
import traceback
import sys, os
from datetime import datetime

currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)

from intelli_utilities import BaiException

parser = argparse.ArgumentParser()
parser.add_argument('--port', help='socket port to use')
args = parser.parse_args()

import logging
logging.basicConfig(filename='log/socket.log', encoding='utf-8', level=logging.ERROR)

def run_script(data):
    req_data = json.loads(data)
    args = req_data['arguments']
    """
    All set variable in 'scriptFile' will be avaiable in data variable as dict key.
    Which means 'scriptFile' must set a variable named response for return data.
    Set a variable named 'error' to return error statements
    """
    data = runpy.run_path(path_name='filedb/scripts/{}/{}'.format(req_data['scriptType'], req_data['scriptFile']), run_name='__main__', init_globals={'runpy_args': args})
    return data['response']


def divide_into_chunks(input_string, chunk_size=16384):
    chunks = []
    for i in range(0, len(input_string), chunk_size):
        chunk = input_string[i:i + chunk_size]
        chunks.append(chunk)
    return chunks


def start_server():
    response = {
        'data': None,
        'statusCode': HTTPStatus.OK,
        'error': None
    }            
            
    HOST = '127.0.0.1'
    PORT = int(args.port)
    
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.bind((HOST, PORT))
        server_socket.listen()
        conn, _ = server_socket.accept()
        with conn:
            while True:
                data = conn.recv(16384)
                try:
                    response['data'] = run_script(data)
                    response['statusCode'] = HTTPStatus.OK
                    response['error'] = None
                except BaiException as be:
                    response['statusCode'] = HTTPStatus.INTERNAL_SERVER_ERROR
                    now = datetime.now()
                    error = "{}: {}".format(now.strftime('%Y-%m-%dT%H:%M:%S%z'),be)
                    response['error'] = error
                except Exception as e:
                    response['statusCode'] = HTTPStatus.INTERNAL_SERVER_ERROR
                    exc_type, _, exc_tb = sys.exc_info()
                    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]        
                    now = datetime.now()
                    error = "{}: {} {} {} {} {}".format(now.strftime('%Y-%m-%dT%H:%M:%S%z'), e, exc_type, fname, exc_tb.tb_lineno, traceback.format_exc())
                    response['error'] = error
                    logging.error(error)                    

                chunks = divide_into_chunks(json.dumps(response))
                for chunk in chunks:
                    conn.send(chunk.encode())

if __name__ == "__main__":
    start_server()