from unittest import  TestCase, main

import pandas as pd
# from datetime import datetime

from intelli_utilities import *
from DataProcessor import DataProcessor


class TestIntelliUtilities(TestCase):
    
    def test_convert_histdata_to_dataframe(self):
        
        # Test hour interval autumn CEST / CET
        input_dms_hist_data = [
            {'2023-10-29T01:00:00,000+02:00': 23}, 
            {'2023-10-29T02:00:00,000+02:00': 44},
            {'2023-10-29T02:00:00,000+01:00': 21},
            {'2023-10-29T03:00:00,000+01:00': 46},
            {'2023-10-29T04:00:00,000+01:00': 11},
            {'2023-10-29T05:00:00,000+01:00': 36},
        ]
        
        expected_index = [
            '2023-10-29 01:00:00+02:00',
            '2023-10-29 02:00:00+02:00',
            '2023-10-29 02:00:00+01:00',
            '2023-10-29 03:00:00+01:00',
            '2023-10-29 04:00:00+01:00',
            '2023-10-29 05:00:00+01:00',
        ]        

        token = ''
        base_url = ''
        dp  = DataProcessor(token, base_url)
        df  = dp.convert_histdata_to_dataframe(input_dms_hist_data)

        self.assertIsInstance(df, pd.DataFrame)
        self.assertEqual('value', df.columns[0], 'First column name must be value')

        index_list = [str(val) for val in df.index]
        self.assertListEqual(index_list, expected_index)           
                
        df  = dp.convert_histdata_to_dataframe(input_dms_hist_data, 'new_col_name')
        self.assertIsInstance(df, pd.DataFrame)
        self.assertEqual('new_col_name', df.columns[0], 'First column name must be new_col_name')        
     
    # def test_skip_cron(self):        
        
    #     # Test hourly
    #     data = {
    #         'input': {
    #             'interval': 3600, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 15, 0)
    #         },
    #         'expected': False,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])
        
    #     data = {
    #         'input': {
    #             'interval': 3600, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 11, 20)
    #         },
    #         'expected': True,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])
        
        
    #     data = {
    #         'input': {
    #             'interval': 3600, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 22, 1)
    #         },
    #         'expected': True,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])
                
    #     # Test daily
    #     data = {
    #         'input': {
    #             'interval': -1, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 22, 1)
    #         },
    #         'expected': True,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])        
        
    #     data = {
    #         'input': {
    #             'interval': -1, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 22, 1)
    #         },
    #         'expected': True,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])    
               
    #     data = {
    #         'input': {
    #             'interval': -1, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 2, 11)
    #         },
    #         'expected': True,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])
        
    #     data = {
    #         'input': {
    #             'interval': -1, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 22, 00)
    #         },
    #         'expected': False,
    #     }        
    #     result = skip_cron(data['input']['interval'], data['input']['time'], data['input']['now_obj'])    
    #     self.assertEqual(result, data['expected'])          
        
    #     # Test none existing interval
    #     data = {
    #         'input': {
    #             'interval': -3, 
    #             'time': '22:00',
    #             'now_obj': datetime(2022, 3, 2, 22, 00)
    #         },
    #         'expected': Exception,
    #     }        
    #     self.assertRaises(data['expected'], skip_cron, data['input']['interval'], data['input']['time'], data['input']['now_obj'])               
    
    
    def test_get_closest_index(self):
        
        data = {
            'input': {
                'value': -9,
                'bin_range': [-10, -5, 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50],
            },
            'expected': 0
        }
        
        idx = get_closest_index(data['input']['value'], data['input']['bin_range'])
        self.assertEqual(idx, data['expected'])
        self.assertIsInstance(idx, int)
        
        data = {
            'input': {
                'value': 46,
                'bin_range': [-10, -5, 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50],
            },
            'expected': 11
        }   
        idx = get_closest_index(data['input']['value'], data['input']['bin_range'])
        self.assertEqual(idx, data['expected'])
        
        
        
    def test_get_bin_range(self):            
        data = {
            'input': {
                'base': 0,
                'step': 5,
                'min_val': -10,
                'max_val': 50,
            },
            'expected': json.dumps([-10, -5, 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50])
        }
        
        range = get_bin_range(data['input']['base'], data['input']['step'], data['input']['min_val'], data['input']['max_val'])
        self.assertEqual(json.dumps(range), data['expected'])        

        data = {
            'input': {
                'base': 0,
                'step': 5,
                'min_val': -10,
                'max_val': 50,
            },
            'expected': json.dumps([-10, -5, 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50])
        }
        range = get_bin_range(data['input']['base'], data['input']['step'], data['input']['min_val'], data['input']['max_val'])
        self.assertEqual(json.dumps(range), data['expected'])
        
                
        data = {
            'input': {
                'base': 0,
                'step': 0,
                'min_val': -10,
                'max_val': 50,
            },
            'expected': BaiException
        }       
        self.assertRaises(data['expected'], get_bin_range, data['input']['base'], data['input']['step'], data['input']['min_val'], data['input']['max_val'])                        
        
    
    # def test_create_datetimes(self):
        
    #     exptected_empty_end_date = datetime.now().replace(hour=0, minute=0, second=0).strftime('%Y-%m-%d %H:%M')
    #     exptected_empty_end_date = datetime.fromisoformat(exptected_empty_end_date).astimezone(LOCAL_TIME_ZONE).strftime('%Y-%m-%dT%H:%M:%S%z')
        
    #     data  = [
    #         {
    #             'input': {
    #                 'dates' : [
    #                     {
    #                         'start': '2022-01-01',
    #                         'end': '2022-01-14',        
    #                     },                     
    #                 ],
    #                 'time' : '00:00'
    #             },
    #             'expected_rv': [
    #                 {
    #                     'start': '2022-01-01T00:00:00+0100',
    #                     'end': '2022-01-14T00:00:00+0100',         
    #                 },   
    #             ],
    #         },
    #         {
    #             'input': {
    #                 'dates' : [
    #                     {
    #                         'start': '2022-01-01',
    #                         'end': '',        
    #                     },                     
    #                 ],
    #                 'time' : '00:00'
    #             },
    #             'expected_rv': [
    #                 {
    #                     'start': '2022-01-01T00:00:00+0100',
    #                     'end': exptected_empty_end_date,
    #                 },   
    #             ],
    #             'errMsg': 'End date should be today'            
    #         }            
    #     ]
       
    #     for idx, item in enumerate(data):
    #         dates = create_datetimes(item['input']['dates'], item['input']['time'], 3600)        
    #         errMsg = item['errMsg'] if 'errMsg' in item else 'error at index {} '.format(idx)
    #         self.assertEqual(json.dumps(dates), json.dumps(item['expected_rv']), errMsg)
        
        
        
    def test_serialize_interpolation(self):
        interpolation_no_method = {
            'template': 'rawValues'
        }
        
        serialized_interpolation = serialize_interpolation(interpolation_no_method)
        self.assertEqual('', serialized_interpolation, 'missing method shoud return empty string')

        interpolation_full = {
            'template': 'rawValues',
            'method': {
                'diff': 'diff',
                'name': 'prevNext',
                'type': 'Linear',
                'fill': 'Fill',
                'statistic': 'Average',
                
            }
        }        
        
        serialized_interpolation = serialize_interpolation(interpolation_full)
        self.assertEqual('diffprevNextLinearFillAverage', serialized_interpolation)
        
        interpolation_no_stat_diff = {
            'template': 'rawValues',
            'method': {
                'name': 'prevNext',
                'type': 'Linear',
                'fill': 'FillNull',
                
            }
        }              
        
        serialized_interpolation = serialize_interpolation(interpolation_no_stat_diff)
        self.assertEqual('prevNextLinearFillNull', serialized_interpolation)        
        
        empty_interpolation = {}
        serialized_interpolation = serialize_interpolation(empty_interpolation)
        self.assertEqual('', serialized_interpolation)        

        
    
if __name__ == '__main__':
    main()