from datetime import timedelta

import os
import sys
currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)

sys.path.insert(0, parentdir)
from sklearn.utils import all_estimators

from mlinsights.mlmodel import PiecewiseRegressor
import xgboost as xgb
import jsonpickle


def get_all_regression_estimators():
        
    skip = [
        'CCA',
        'MultiTaskElasticNet',
        'MultiTaskElasticNetCV',
        'MultiTaskLasso',
        'MultiTaskLassoCV',
        'TweedieRegressor',
        'SGDRegressor',
        'PoissonRegressor',
        'PassiveAggressiveRegressor',
        'ElasticNet',
        'IsotonicRegression',
        'QuantileRegressor',
        'PLSCanonical',
        'TransformedTargetRegressor',
        'MultiOutputRegressor',
        'RegressorChain',
        'StackingRegressor',
        'VotingRegressor',
        'PLSRegression',
        'RadiusNeighborsRegressor',
    ]
    
    all_regs = {}          
    estimators = all_estimators(type_filter='regressor')
    for class_name, RegClass in estimators:
        if class_name in skip:
            continue
        try:
            all_regs[class_name] = RegClass
        except Exception:
            pass
        
    all_regs['XGBoost'] = xgb.XGBRegressor
    all_regs['PiecewiseRegressor'] = PiecewiseRegressor
    
    return all_regs

def get_model_params(config_model:dict):
    model_params = {}
    task_model = config_model['task']['algorithm']['model']
    if task_model:
        if 'gridSearch' in config_model:
            if task_model == config_model['gridSearch']['model']:
                #  config_model['task']['metric'] is empty when task over job result
                if config_model['task']['metric'] and config_model['gridSearch']['tunedMetric'] < config_model['task']['metric']:
                    if config_model['gridSearch']['bestParams'] != '':
                        model_params = config_model['gridSearch']['bestParams']
                    elif config_model['gridSearch']['bestParamsEncoded'] != '':        
                        model_params = jsonpickle.decode(config_model['gridSearch']['bestParamsEncoded'])
                
    return model_params
            

def get_extinfos(node:dict) -> dict:
    extInfos:dict = {
        'name': '',
        'unit': ''
    }
    if 'extInfos' in node:
        extInfos['name'] = node['extInfos']['name'] if 'name' in node['extInfos'] else ''
        extInfos['unit'] = node['extInfos']['unit'] if 'unit' in node['extInfos'] else ''
    return extInfos


# keeping example to update weekly interval implementation
def compute_weekly_startdate(config_startdate_obj, chart_startdate_obj): 
    if chart_startdate_obj < config_startdate_obj:
        while(chart_startdate_obj < config_startdate_obj):
            config_startdate_obj = config_startdate_obj - timedelta(days=7)

    if chart_startdate_obj > config_startdate_obj:
        while(chart_startdate_obj > config_startdate_obj):
            config_startdate_obj = config_startdate_obj + timedelta(days=7)

    return config_startdate_obj

# def get_grouped_annotations(main_hist_data:dict, thresholds_df:pd.DataFrame, main_node_start:str, main_node_end:str):
#     """
#     If chart resolution does not equal to the computed thresholds interval, we group together close events and increase the size.
#     On zoom in the event groups are split accordingly
#     """
#     points = []
#     main_data_df = convert_histdata_to_dataframe(main_hist_data)

#     # Create timestamps for dist computations
#     main_data_df['ts'] = main_data_df.index.astype('int64') // 10**9
#     thresholds_df['ts_annotation'] = thresholds_df.index.astype('int64') // 10**9

#     # Strip events to main start/end
#     th_stripped_df = thresholds_df.loc[pd.to_datetime(main_node_start).strftime('%Y-%m-%d %H:%M'):pd.to_datetime(main_node_end).strftime('%Y-%m-%d %H:%M')]

#     # Merge them with main node data
#     violations_df = th_stripped_df.query('event_upper == True or event_lower == True or alarm_upper == True or alarm_lower == True')[['ts_annotation', 'event_upper', 'event_lower', 'alarm_upper', 'alarm_lower']]
#     df = main_data_df.join(violations_df, how='outer', sort=True)

#     # Remove all entries before first violation
#     indexes= df[df['ts_annotation'].isna() == False].index.tolist()
#     if len(indexes) == 0:
#         return points        
#     df = df.loc[indexes[0]:]

#     size = 0
#     timestamps = []
#     y_values = []
#     fill_color = 'blue'
#     for _, item in df.iterrows():
#         violation = item[['event_upper', 'event_lower', 'alarm_lower', 'alarm_upper']].notnull().values.any()
#         if violation == True:
#             size += 1
#             timestamps.append(item['ts_annotation'] * 1000)
#             y_values.append(item['value'])
#             if  item['alarm_lower'] == True or  item[ 'alarm_upper'] == True:
#                 fill_color = 'red'
#         else:
#             if len(timestamps) > 0:
#                 middleIndex = round((len(timestamps) -1)  / 2)
#                 point = {
#                     'x': timestamps[middleIndex],
#                     'y': np.median(np.nan_to_num(y_values)),
#                     'marker': {
#                         'size': size+1*4,
#                         'fillColor': fill_color,
#                         'strokeColor': '#fff',
#                         'strokeWidth': 1,
#                     },
#                 }
#                 fill_color = 'blue'
#                 points.append(point)
#                 size = 0
#                 timestamps = []
#                 y_values = []
#     return points