import os
import sys
import inspect

# Import intelli_utilities
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)

from intelli_utilities import *

import json
import traceback
from utils import *
from time_slot_monitor import *

if __name__ == '__main__':
  """
  import intelli_utilities will allow you to access the following variables:
  - page: page configuration as dict
  - data_list: list of nodes data
  - config: dict of script configuration
  """
  try:
    # Fetch main node data
    main_node = None
    for node in page['nodes']:
        if node['uuid'] == args.node_uuid:
            main_node = node
            break

    #Quick fix to handle amount of datas
    match main_node['interval']:
        case -1:
            start_date = "2018-01-04T21:00:00Z"
        case _:
            start_date = "2022-06-01T00:00:00Z"


    hist_data = fetch_hist_data(main_node, args.token, baseUrl, start_date)

    # Remove outliers, negative values, drops NA and round decimals
    hist_data_df = convert_histdata_to_dataframe(hist_data)

    # Get script params
    sensitivity = configModel['sensitivity']
    monitoring_time_start = configModel['monitoringTimeStart']
    monitoring_time_end = configModel['monitoringTimeEnd']
    min_th = float(configModel['minThreshold'])
    day_event_count = configModel['dailyEventCount']
    week_event_count = configModel['weeklyEventCount']

    configMap = get_config_mappig(sensitivity)
    lookback = configMap['lookback']
    th_percent = configMap['thresholdPercent']
    alpha = configMap['alpha']

    df = preprocessing(hist_data_df, time_col='x', target_col='y')
    ad = TimeSlotMonitor(target_col='y', lookback=lookback, th_percent=th_percent, min_th=min_th, alpha=alpha, monitoring_time_start=monitoring_time_start, monitoring_time_end=monitoring_time_end, verbose=False)
    ad.fit(df[0:50])
    pred = ad.predict(df)

    pred['dateTime'] = pred['dateTime'].astype(str)

    # Extract events and alarms from df
    events = pred[['dateTime','event']][pred['event'] == 1]
    # alarms = pred[['dateTime','alarm']][pred['alarm'] > 0]

    # Columns get some name for more convenience
    events = events.rename(columns={'dateTime': 'x', 'event': 'y'}).to_dict('records')

    threshold = pred[['dateTime','th']]
    threshold = threshold.round(1)
    threshold.rename(columns={'dateTime': 'x', 'th': 'y'}, inplace=True)
    threshold = threshold.to_dict('records')

    time_slot_mean = pred[['dateTime','time_slot_mean']]
    time_slot_mean = time_slot_mean.round(1)
    time_slot_mean.rename(columns={'dateTime': 'x', 'time_slot_mean': 'y'}, inplace=True)
    time_slot_mean = time_slot_mean.to_dict('records')

    annotations = {}
    if len(events):
      annotations = get_event_annotations(events, day_event_count, week_event_count, monitoring_time_start, monitoring_time_end)

    output = json.dumps({
        'series': [
            { 'name': 'monitoringTimeMean', 'data': time_slot_mean, 'color': '#ED4416' },
            { 'name': 'threshold', 'data': threshold, 'color': '#167BED' }
        ],
        'options': {
            'chart' : {
                'type': 'line',
            },
            'annotations': annotations,
            'yaxis': [
                {
                    'show': False,
                    'seriesName': 'monitoringTimeMean'
                },
                {
                    'show': False,
                    'seriesName': 'threshold'
                },
            ]
        }
    })    
  except Exception as e:
    output = json.dumps({'error': traceback.format_exc()})
  print(output) # return output