#!/bin/bash

# Function to check if the user has sudo privileges
check_sudo() {
  if sudo -n true 2>/dev/null; then
    echo "You have sudo privileges."
  else
    echo "This script requires sudo privileges."
    echo "Please enter your password to continue."
    
    # Prompt the user for sudo access
    if ! sudo -v; then
      echo "Failed to acquire sudo privileges. Exiting."
      exit 1
    fi
  fi
}

check_sudo

# Detect the distribution by reading /etc/os-release
. /etc/os-release

if [ "$ID" == "debian" ]; then
  echo "Detected Debian..."
  echo "Updating repositories..."
  sudo apt update

  # Install Python 3.11 and python3.11-venv (available in Debian Bookworm's official repository)
  echo "Installing Python 3.11 and python3.11-venv..."
  sudo apt install python3.11 python3.11-venv -y

elif [ "$ID" == "ubuntu" ]; then
  echo "Detected Ubuntu-based system..."
  echo "Updating repositories..."
  sudo apt update
  sudo apt install software-properties-common -y

  # Add deadsnakes PPA for Python 3.11 (if not already available in the official repository)
  echo "Adding deadsnakes PPA for Python 3.11..."
  sudo add-apt-repository ppa:deadsnakes/ppa -y
  sudo apt update

  # Install Python 3.11 and python3.11-venv
  echo "Installing Python 3.11 and python3.11-venv..."
  sudo apt install python3.11 python3.11-venv -y
else
  echo "Unsupported distribution. This script works on Ubuntu and Debian-based systems."
  exit 1
fi

# Create a directory for Python 3.11 and set a symbolic link for convenience
echo "Setting up symbolic link for python3.11..."
sudo mkdir -p $(dirname "$0")/python311
sudo ln -sf /usr/bin/python3.11 $(dirname "$0")/python311/python

echo "Python 3.11 installation completed successfully."

echo "Press any key to exit..."
read -n1 -s  # Waits for one key press without echoing it to the terminal
