// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
const StatusTable = window.nguiAsyncImport('public/ui/main/process-manager/instances/processes/status/index.js')

window.ngui(() => {
  return {


    beforeCreate() {
      components.setParent(this)
    },

    // Listen on mounted event...
    created() {
      // logic
      if (this.processManagerUuid || this.$attrs.serviceUuids) {
        this.ctx.endpoints.setServiceUuids(this.$attrs.serviceUuids || [this.processManagerUuid])
      }
      this.instance = this.$attrs.instance
      this.processInstance = this.$attrs.processInstance
      this.process = this.$attrs.process
      this.dmsEndpoint = this.ctx.routing.getHttpUrl(this.$attrs.dmsEndpoint)
      this.init()
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processManagerUuid: this.$route.query.service,
        instance: '',
        processInstance: '',
        process: '',
        loading: true,
        deviceConfigurations: [],
        settings: {
          deviceConfigurationName: '',
        },    
      }
    },

    // Define methods for the ui...
    methods: {
      async init() {
        this.loading = true
        this.fetchDeviceConfigurations()
        this.settings = this.ctx.rest.lsGetJSON('genStx:settings') || this.settings
      },

      fetchDeviceConfigurations() {
        this.ctx.httpClient.get(this.ctx.endpoints.getHttp(`/service/${this.ctx.endpoints.getLocalService()}/v1/configurations/${this.process}/configuration/Devices`))
          .then((res) => {
            this.deviceConfigurations = res.data.content || []
          })
          .catch((err) => {
            components.notify().negative(err)
          })
          .finally(() => {
            this.loading = false
          })
      },

      getGenerateDeviceOptions() {
        const options = []
        this.deviceConfigurations.forEach((deviceConfiguration) => {
          if (deviceConfiguration.enabled && deviceConfiguration.instance === this.instance) {
            options.push({ value: deviceConfiguration.name, label: deviceConfiguration.name, caption: deviceConfiguration.description })
          }
        })
        return options
      },

      generate() {
        this.ctx.rest.lsSetJSON('genStx:settings', this.settings)
        const configuration = this.deviceConfigurations.find(obj => {
          return obj.name === this.settings.deviceConfigurationName
        })
        if (configuration.name) {
          this.ctx.httpClient.post(this.ctx.endpoints.getHttp('/service/GenSTX/v1/platforms/generatePLC'), {
              name: configuration.name,
              description: configuration.description,
              applications: configuration.applications,
              platform: configuration.platform,
              ip: configuration.platformSettings.ip,
              subnet: configuration.platformSettings.subnet,
              gateway: configuration.platformSettings.gateway,
              port: configuration.platformSettings.port,
              plcType: configuration.platformSettings.type,
              protocol: configuration.platformSettings.protocol,
              libraryName: configuration.platformSettings.libName,
              libraryVendor: configuration.platformSettings.libVendor,
              libraryVersion: configuration.platformSettings.libVersion,
              source: {
                apiType: "DMS",
                endpoint: this.dmsEndpoint,
              }
            })
            .then((res) => {
              components.notify().positive(res)
            })
            .catch((err) => {
              components.notify().negative(err)
            })
        }
        else {
          components.notify().negative('invalid configuration')
        }
      },
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.element('NgSection').attrs({ permissions: [31] }).append(() => [
          // components.loading('loading', this.loading, (val) => this.loading = val).attrs(),
          components.form().attrs({ 
            onSubmit: () => this.generate(), 
            submitLabel: 'generate', 
            disable: !this.settings.deviceConfigurationName,
          }).append(() => [
            components.input('device', 'select', this.settings.deviceConfigurationName, (val) => this.settings.deviceConfigurationName = val)
              .attrs({ required: true, options: this.getGenerateDeviceOptions() }),
          ]),
          components.element(StatusTable).attrs({
            class: 'q-mt-md',
            name: 'GenSTX',
            serviceUuids: this.$attrs.serviceUuids,
            processManager: this.processManagerUuid,
            instance: this.processInstance,
            process: this.process,
            statusIds: [''],
          })
        ])
      ]
    },
  }
})