// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: ['workspaceUuid', 'serviceUuid'],
    emits: ['submit', 'cancel'],
    beforeCreate() {
      components.setParent(this)
    },

    // Listen on mounted event...
    mounted() {
      // logic
      if (this.workspaceUuid) {
        this.ctx.workspace.toService([this.processManagerUuid]).fetchWorkspace(this.workspaceUuid)
          .then((workspace) => {
            this.form = workspace
          })
      } else {
        this.form.uuid = this.ctx.uuidv4()
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processManagerUuid: this.serviceUuid || this.$route.query.service,
        form: {
          name: '',
          description: '',
          uuid: '',
          instances: [],
          promosnts: [],
        },
        promosnt: {
          name: '',
          url: 'ws://localhost:9020/json_data',
          uuid: '',
          type: 'DMS',
        },
        ntOptions: [
          {
            label: 'DMS',
            value: 'DMS',
          },
          {
            label: 'pWebAccess',
            value: 'pWebAccess',
          },
        ]
      }
    },

    // Define methods for the ui...
    methods: {
      submit() {
        this.ctx.workspace.toService([this.processManagerUuid]).postWorkspace(this.form)
            .then((res) => {
              this.$emit('submit', res)
            })    
      },
      cancel() {
        this.$emit('cancel')
      },

      addNtEndpoint() {
        this.promosnt.uuid = this.ctx.uuidv4()
        if (!this.form.promosnts) {
          this.form.promosnts = []
        }
        this.form.promosnts.push(JSON.parse(JSON.stringify(this.promosnt)))
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.form().append(() => [
          components.caption('general'),
          components.input('ID', 'text', this.form.uuid, (val) => this.form.uuid = val).attrs({ readonly: true }),
          components.input('name', 'text', this.form.name, (val) => this.form.name = val).attrs({ required: true }),
          components.input('description', 'text', this.form.description, (val) => this.form.description = val),
          components.caption('projects'),
          components.element('NgWorkspaceSelectInstances').attrs({
            modelValue: this.form.instances,
            serviceUuids: [this.processManagerUuid],
            'onUpdate:modelValue': (val) => this.form.instances = val,
          }),

          components.caption('ProMoS NT').attrs({ class: 'q-mt-md' }),
          
          components.list().append(() => [
            components.item('endpoints', this.form.promosnts ? this.form.promosnts.length : 0, true).append(() => [
              
              components.header().append(() => [
                components.button('add', () => this.addNtEndpoint()),
              ]),
              
              ...this.form.promosnts.map((nt, idx) => components.item().append(() => [
                components.itemSection().attrs({ class: 'q-mb-sm' }).append(() => [
                  components.label().append(() => [
                    components.input('name', 'text', this.form.promosnts[idx].name, (val) => this.form.promosnts[idx].name = val),
                  ]),
                  components.label().append(() => [
                    components.input('url', 'url', this.form.promosnts[idx].url, (val) => this.form.promosnts[idx].url = val),
                  ]),
                  components.label().append(() => [
                    components.input('type', 'select', this.form.promosnts[idx].type, (val) => this.form.promosnts[idx].type = val).attrs({
                      options: this.ntOptions,
                    }),
                  ]),

                  components.separator(),
                ]),
                components.itemSection().attrs({ side: true, top: true }).append(() => [
                  components.button('delete', () => {
                    this.form.promosnts.splice(idx, 1);
                  })
                ]),
            
              ])),
            ]),
          ]).attrs({
            bordered: true,
          }),

        ]).on({
          onSubmit: () => this.submit(),
          onCancel: () => this.cancel(),
        })
      ]
    },
  }
})