import useUtils from '/service/rest/v1/files/data/public/js/useUtils.js'
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'

const components = useComponent()

export default function useProcessManager() {

  const utils = useUtils()
  
  let parent = null
  
  function setParent(p) {
    parent = p
    components.setParent(p)
  }

  function mixin() {
    return {
      data() {
        return {
          endpoint: '',
          selectedManagers: [],
          selectedInstances: [],
          selectedWorkspaces: [],
          selectedProcesses: [],
          template: [],
          processManagers: [],
          processManager: {
            uuid: '',
            name: '',
            service: '',
            options: {
              name: '',
            }
          },
          instancesSettings: [],
          instancesStatus: [],
          processesSettings: [],
          processesStatus: [],
          instanceSettings: {
            uuid: '',
            name: ''
          },
          instanceStatus: {
            uuid: '',
            name: '',
            processes: [],
          },
          processSettings: {
            uuid: '',
            name: '',
            arguments: [],
          },
          processStatus: {
            uuid: '',
            name: ''
          },
          instanceServers: {
            processes: [],
          },
          processServer: {
            location: {
              hostName: '',
              hostIp: '',
              port: 0,
            },
          },
          processHasModule: false,
          workspaces: [],
          workspace: {
            uuid: '',
            name: '',
            description: '',
            instances: [],
            roles: [
              '*'
            ],
            promosnts: [],
          },
        }
      },
      methods: {
        removeSelectedManagers() {
          const remove = this.selectedManagers.filter((item) => item.service !== 'local-process-manager')
          const promise = Promise.all(remove.map((x) => this.ctx.processManager.deleteManager(x.service)))
          this.selectedManagers = []
          return promise
        },

        removeSelectedInstances(processManagerUuid) {
          const remove = this.selectedInstances.filter((item) => item.name !== '_ProcessManager')
          const promise = Promise.all(remove.map((x) => this.ctx.processManager.deleteInstance(processManagerUuid, x.uuid)))
          this.selectedInstances = []
          return promise
        },

        removeSelectedProcesses(processManagerUuid, instanceUuid) {
          const remove = this.selectedProcesses.filter((item) => item.name !== 'ProcessManager')
          const promise = Promise.all(remove.map((x) => this.ctx.processManager.deleteProcess(processManagerUuid, instanceUuid, x.uuid)))
          this.selectedProcesses = []
          return promise
        },

        removeSelectedWorkspaces(processManagerUuid) {
          const promise = Promise.all(this.selectedWorkspaces.map((x) => this.ctx.workspace.toService([processManagerUuid]).deleteWorkspace(x.uuid)))
          this.selectedWorkspaces = []
          return promise
        }
      }
    }
  }

  function getPMBreadcrumbs(processManagerUuid, instanceUuid, processUuid, last) {
    if (Array.isArray(processManagerUuid)) {
      return processManagerUuid
    }
    const breadcrumbs = [{
      label: parent.ctx.i18n.i18n.t('managers'),
      to: '/process-manager',
      icon: 'public',
      tooltip: 'processManagers',
    }]

    if (processManagerUuid) {
      breadcrumbs.push({
        label: parent.processManager.options.name || parent.ctx.i18n.i18n.t('manager'),
        to: `/process-manager/index?service=${parent.$route.query.service || ''}`,
        icon: 'location_city',
        tooltip: 'instances',
      })
    }

    if (instanceUuid) {
      breadcrumbs.push({
        label: parent.instanceSettings.name || parent.instanceStatus.name || parent.ctx.i18n.i18n.t('project'),
        to: `/process-manager/instances/index?service=${parent.$route.query.service || ''}&instance=${instanceUuid}`,
        icon: 'home_filled',
        tooltip: 'processes',
      })
    }

    if (processUuid) {
      breadcrumbs.push({
        label: parent.processSettings.name || parent.processStatus.name || parent.ctx.i18n.i18n.t('process'),
        to: `/process-manager/instances/processes/index?service=${parent.$route.query.service || ''}&instance=${instanceUuid}&process=${processUuid}`,
        icon: 'widgets',
      })
    }

    if (last) {
      breadcrumbs.push({ label: parent.ctx.i18n.i18n.t(last) })
    }


    return breadcrumbs
  }

  function getPMWorkspaceBreadcrumb() {
    return [
      {
        label: parent.ctx.i18n.i18n.t('workspaces'),
        to: '/workspaces',
        icon: 'fas fa-layer-group',
        tooltip: 'workspaces',
      },
      {
        label: parent.ctx.workspace.ctx.workspace.name || parent.ctx.i18n.i18n.t('workspace'),
        to: `/workspaces/index?service=${parent.$route.query.service || ''}&workspace=${parent.ctx.workspace.ctx.workspace.uuid}`,
        icon: 'fas fa-industry',
        tooltip: 'workspace',
      },
    ]
  }

  function getPlayMenu(status, permissions = { update: [], delete: [] }) {
    let menus = []
    const disable = status.name === '_ProcessManager' || status.name === 'ProcessManager'  
    
    switch (status.statusText) {
      case 'Starting':
      case 'Terminating':
        break
      case 'Running':
        menus.push({ id: 'Stop', label: 'stop', icon: 'stop', disable }, { id: 'Restart', label: 'restart', icon: 'replay' })
        break
      case 'Scheduled':
      case 'RunOnce':
        menus.push({ id: 'Execute', label: 'execute', icon: 'play_arrow' }, { id: 'Stop', label: 'stop', icon: 'stop', disable }, { id: 'Restart', label: 'restart', icon: 'replay' })
        break
      case 'NotRunning':
        menus.push({ id: 'Start', label: 'start', icon: 'play_arrow' })
        break
      default:
        menus.push({ id: 'Start', label: 'start', icon: 'play_arrow' }, { id: 'Stop', label: 'stop', icon: 'stop', disable }, { id: 'Restart', label: 'restart', icon: 'replay' })
        break  
    }

    menus.forEach((menu) => {
      menu.permissions = permissions.update
    })

    return menus
  }

  function getActionMenu(status, permissions = { update: [], delete: [] }) {
    let menus = getPlayMenu(status, permissions)
    const disable = status.name === '_ProcessManager' || status.name === 'ProcessManager'  

    // Add separator to last item
    if (menus.length >= 1) {
      menus[menus.length - 1].separator = true
    }

    // Push edit and delete action
    menus.push(
      {
        id: 'edit',
        label: 'edit',
        icon: 'edit',
        permissions: permissions.update,
      },
      {
        id: 'delete',
        label: 'delete',
        icon: 'delete',
        disable,
        permissions: permissions.delete,
      },
    )

    return menus 
  }

  function getActionButtons(status, callback = () => {}) {
    const buttons = []
    const disable = status.name === '_ProcessManager' || status.name === 'ProcessManager'  
    switch (status.statusText) {
      case 'Starting':
      case 'Terminating':
        break
      case 'Running':
        buttons.push(
          components.button('stop', 'stop', 'negative').on({
            onClick: () => callback('Stop'),
            disable,
          }),
          components.button('restart', 'replay', 'warning').on({
            onClick: () => callback('Restart')
          }),
        )
        break
      case 'Scheduled':
      case 'RunOnce':
        buttons.push(
          components.button('execute', 'play_arrow', 'blue-7').on({
            onClick: () => callback('Execute')
          }),
          components.button('stop', 'stop', 'negative').on({
            onClick: () => callback('Stop'),
            disable,
          }),
          components.button('restart', 'replay', 'warning').on({
            onClick: () => callback('Restart')
          }),
        )
        break
      case 'NotRunning':
        buttons.push(
          components.button('start', 'play_arrow', 'positive').on({
            onClick: () => callback('Start')
          }),
        )
        break
      default:
        buttons.push(
          components.button('start', 'play_arrow', 'positive').on({
            onClick: () => callback('Start')
          }),
          components.button('stop', 'stop', 'negative').on({
            onClick: () => callback('Stop'),
            disable,
          }),
          components.button('restart', 'replay', 'warning').on({
            onClick: () => callback('Restart')
          }),
        )
        break  
    }

    buttons.forEach((button) => {
      button.attrs({
        size: 'sm',
      })
    })

    return components.element('NgSection').attrs({ class: 'flex q-gutter-sm q-mb-sm' }).append(() => buttons)
  }
  
  function getStatusSubtitle(status) {
    return status.status === 2 || status.status === 5 ? 
      `Uptime: ${utils.getDurationString(status.upTime)}, CPU-Time: ${utils.getDurationString(status.cpuTime)}, CPU: ${status.cpuUsage.toFixed(2)}%, MEM: ${utils.formatBytes(status.memUsage)}` :
      ''
  }

  function getStateColor(status) {
    let color = ''
    switch (status.statusText) {
      case 'NotRunning':
        color = status.enabled ? 'orange-4' : 'grey'
        break
      case 'Starting':
        color = 'green-4'
        break
      case 'Running':
        color = 'positive'
        break
      case 'Terminating':
        color = 'orange-4'
        break
      case 'Error':
        color = 'negative'
        break
      case 'PartialRunning':
      case 'Scheduled':
      case 'RunOnce':
      case 'Executing':
        color = 'blue-7'
        break
      default:
        color = 'grey'
        break;
    }
    return color
  }

  function getStatusIcon(status) {
    if (status.enabled && status.statusText === 'NotRunning') {
      return 'stop'
    }

    if (status.statusText === 'Error') {
      return 'notifications'
    }

    if (status.statusText === 'Terminating' || status.statusText === 'Starting') {
      return 'hourglass_bottom'
    }

    if (!status.enabled) {
      return 'block'
    }

    return 'check'
  }

  function isStatusRunning(status) {
    return status.status === 2 || status.status === 5  
  }

  function getStatusText(status) {
    return `${status.statusText}${status.startTermProgress >= 0 && status.startTermProgress < 100 ? ` (${status.startTermProgress}%)` : ''}`
  }
  
  function getStatusComponents(status, gridName = 'flex', disableExtInfos = false) {
      return components.element('div').attrs({ class: `${gridName} justify-start q-gutter-sm items-center help-status-badges`, }).append(() => [
        components.badge(getStatusText(status), status.status === 2 ? 'done' : 'block', getStateColor(status)),
        components.badge('autoStart', status.enabled ? 'done' : 'block', status.enabled ? 'positive' : 'grey'),
        isStatusRunning(status) && !disableExtInfos ? components.badge(`Uptime: ${utils.getDurationString(status.upTime)}`, 'timer') : '',
        isStatusRunning(status) && !disableExtInfos ? components.badge(`CPU: ${status.cpuUsage.toFixed(2)}%`, 'monitor_heart') : '',
        isStatusRunning(status) && !disableExtInfos ? components.badge(`MEM: ${utils.formatBytes(status.memUsage)}`, 'memory') : '',
      ])
  }

  function getStatusBadge(status) {
      return components.badge(getStatusText(status), status.enabled ? 'done' : 'block', getStateColor(status))
  }

  function getMonitoringBadge(monitoring = { service: '', options: { error: '', checkedAt: 0 }} ) {
    if (!monitoring.service) {
      return components.badge('Ping', 'network_ping', 'warning')
    }
    
    const hasError = !monitoring || !monitoring.options || monitoring.options.error
    const color = hasError ? 'negative' : 'positive'
    const icon = hasError ? 'block' : 'done'
    const text = hasError ? monitoring.options.error || 'error' : 'ok'
    return components.badge(text, icon, color)
  }

  function isDisableAction(status) {
    return status.statusText === 'Terminating' || status.statusText === 'Starting'
  }

  function getStatusBanners(status) {
    return components.column().attrs({ class: 'q-my-sm full-width' }).append(() => [
      components.renderIf(status.status !== 2, components.bannerWarn(status.statusText).attrs({ class: 'full-width' })),
      components.renderIf(status.status === 4, components.bannerNegative(status.statusText).attrs({ class: 'full-width' })),
    ])
  }


  function hasModule(name = '') {
    const modules = window.ngModules || { data:  [] }
    return !!modules.data.find((module) => module.name === name)
  }

  function hasConfig(name = '') {
    const configs = window.ngConfigs || { data:  [] }
    return !!configs.data.find((config) => config.name === name)
  }

  function toModule(name, processManagerUuid, instanceUuid, processUuid) {
    components.router().push(`/modules/${name}/index`, { 
      service: processManagerUuid,
      instance: instanceUuid,
      process: processUuid,
    })
  }

  function toStatus(processManagerUuid, instanceUuid, processUuid) {
    components.router().push('/process-manager/instances/processes/status/index', { 
      service: processManagerUuid,
      instance: instanceUuid,
      process: processUuid,
    })
  }

  function toLogs(processManagerUuid, instanceUuid, processUuid) {
    components.router().push('/process-manager/instances/processes/logs/index', { 
      service: processManagerUuid,
      instance: instanceUuid,
      process: processUuid,
    })
  }

  function toConfiguration(processManagerUuid, instanceUuid, processUuid, name, index) {
    components.router().push('/process-manager/instances/processes/configuration', { 
      service: processManagerUuid,
      instance: instanceUuid,
      process: processUuid,
      name,
      hasSettings: true,
      bread: index,
    })
  }

  function to(path, processManagerUuid, instanceUuid, processUuid, name, index) {
    components.router().push(path, { 
      service: processManagerUuid,
      instance: instanceUuid,
      process: processUuid,
      name,
      hasSettings: true,
      bread: index,
    })
  }

  return {
    setParent,
    mixin,
    getPMBreadcrumbs,
    getPMWorkspaceBreadcrumb,
    getPlayMenu,
    getActionMenu,
    getActionButtons,
    getStatusSubtitle,
    getStatusComponents,
    getStatusBadge,
    getMonitoringBadge,
    getStateColor,
    getStatusIcon,
    isDisableAction,
    getStatusBanners,
    hasModule,
    hasConfig,
    toModule,
    toStatus,
    toLogs,
    toConfiguration,
    to,
  }
}