// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {

    beforeCreate() {
      components.setParent(this)
    },

    mounted() {
    }, 
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        fileData: {},
        apps: [],
        loading: false,
      }
    },

    // Define methods for the ui...
    methods: {
      // functions
      async fetchApps() {
        this.loading = true
        this.apps = []

        try {
          const res = await this.ctx.httpClient.get('/service/rest/v1/filedb/cfg/modules.json?tryUserDir=true')
          this.fileData = res.data.data.data
        } catch (e) {
          // nothing to do
          components.notify().negative(e)
        }

        const promises = []
        this.fileData.apps.forEach((app) => {
          promises.push(this.ctx.httpClient.get(app.href))
        })
        
        Promise.allSettled(promises).then((results) => {
          results.forEach((result, index) => {
            const app = this.fileData.apps[index]
            if (app.to ||(app.href && result.status !== 'rejected')) {
              this.apps.push(app)
            }
          })              
        })
        
        this.loading = false
      },
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page(() => this.fetchApps()).append(() => [
          window.components.section().attrs({ class: 'full-width' }).append(() => [
            window.components.getButtonsGrid(this.apps),
          ]),

        ])
      ]
    },
  }
})