import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()
 
// Assign components to window to use it where ever you like.
window.components = components;
window.processManager = processManager;

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: ['version'],
    mixins: [
      window.processManager.mixin(),
    ],
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    beforeCreate() {
      window.components.setParent(this);
      window.processManager.setParent(this);
    },

    created() {
      this.ctx.filedb.read('user-data/ui/style.css', true)
        .then((res) => {
          if (res.data.data) {
            window.nguiCreateStyle('user-data-style', res.data.data)
          }
        })
    },

    mounted() {
      this.init()
      this.setMenu()
    }, 
      
    data() {
      return {
        // VAR
        userIsValid: false,
        help: [],
        helpItems: {
          list: [],
        },
        hasHelp: false,
        hasHelpList: false,
        dialog: {
          help: false,
        },
        driver: null,
        loading: true,
        admin: [

        ],
        menus: [],
      }
    },

    // Watch primitive or object values for it state changes...
    watch: {
      // props
      'ctx.routing.route.path'() {
        this.fetchHelp()
      },
    },

    methods: {
      setMenu() {
          this.ctx.globals.init().then(() => {
            this.menus = [
              {
                title: 'apps',
                to: '/',
              },
              {
                title: 'settings',
                to: '/settings',
                permissions: this.ctx.globals.getPermissions(['settingsRead']),
              },
              {
                title: 'workspaces',
                to: '/workspaces',
                permissions: this.ctx.globals.getPermissions(['workspacesRead']),
              },
              {
                title: 'managers',
                to: '/process-manager',
                permissions: this.ctx.globals.getPermissions(['managersRead']),
              },
              {
                title: 'userManagement',
                to: '/users',
                permissions: this.ctx.globals.getPermissions(['usersRead']),
              },
              {
                title: 'Editor',
                to: '/ui-editor',
                permissions: this.ctx.globals.getPermissions(['adminRead']),
              }, 
              {
                title: 'services',
                to: '/services',
                permissions: this.ctx.globals.getPermissions(['adminRead']),
              },
              {
                title: 'Library',
                to: '/service/app/modules/ng-lib',
                type: 'a',
                permissions: this.ctx.globals.getPermissions(['adminRead']),
              },
              {
                title: 'Query',
                to: '/query',
                permissions: this.ctx.globals.getPermissions(['adminRead']),
              },
            ]
          })
      },

      resetHelp() {
        this.help = []
      },
      
      fetchHelp() {
        this.resetHelp()
        if (!this.userIsValid) {
          return
        }
        this.hasHelpList = false
        let path = this.ctx.routing.route.path === '/' ? '/index' : this.ctx.routing.route.path;
        this.ctx.httpClient.get(this.ctx.endpoints.getHttp(`/service/rest/v1/filedb/help${path}.yaml`))
          .then((res) => {
            if (Array.isArray(res.data.data.data)) {
              this.help = res.data.data.data
            } else {
              this.helpList = res.data.data.data
              this.hasHelpList = true
            }
            this.initHelp()
            this.hasHelp = true
          })
          .catch(() => {
            // nothing to do, we do just not display a help button
            this.resetHelp()
            this.hasHelp = false
          })
      },

      initHelp() {
        this.driver = new this.ctx.Driver({
          animate: false,  // Animate while changing highlighted element
          opacity: 0.75,  // Background opacity (0 means only popovers and without overlay)
          padding: 5,    // Distance of element from around the edges
          allowClose: false, // Whether clicking on overlay should close or not
          overlayClickNext: false, // Should it move to next step on overlay click
          doneBtnText: this.ctx.i18n.t('done'), // Text on the final button
          closeBtnText: this.ctx.i18n.t('close'), // Text on the close button for this step
          nextBtnText: this.ctx.i18n.t('next'), // Next button text for this step
          prevBtnText: this.ctx.i18n.t('previous'), // Previous button text for this step
          showButtons: true, // Do not show control buttons in footer
          keyboardControl: true, // Allow controlling through keyboard (escape to close, arrow keys to move)
        })
      },

      startHelp() {
        const steps = this.help.map((help) => ({
            ...help,
            popover: {
              ...help.popover,
              title: this.ctx.i18n.t(help.popover.title),
              description: this.ctx.i18n.t(help.popover.description),
            },
          }))
  
          this.driver.defineSteps(steps)
          // Seems like there is a bug or error in driver start and onClick event. THe onclick event will cancel the process.
          setTimeout(() => {
            this.driver.start()
          }, 100)
      },

      runHelp() {
        if (!this.hasHelpList) {
          this.startHelp()
        } else {
          this.dialog.help = true
        }
      },

      loadHelpFromList(help) {
        this.help = help.items
        this.startHelp()
        this.dialog.help = false
      },

      async fetchMeta() {
        this.loading = true
        this.fetchHelp()
        this.userIsValid = true
        const localWindow = window;
        if (!window.ngModules) {
          const modules = await this.ctx.httpClient.get(this.ctx.endpoints.getHttp('/service/rest/v1/filedb/public/ui/main/modules'))
          localWindow.ngModules = modules.data
        }
        this.loading = false
      },

      init() {
        this.ctx.user.repository.on('login', () => {
          this.fetchMeta()
          this.setMenu()
        })

        this.ctx.user.repository.on('validated', () => {
          this.fetchMeta()
        })
      },

      getColors() {
        return {
          header: this.$q.dark.isActive ? '#0b78e5' : '#0b5db0',
          primary: this.$q.dark.isActive ? '#0b78e5' : '#0b5db0',
          secondary: this.$q.dark.isActive ? '#c4d2dd' : '#7395ae',
          info: this.$q.dark.isActive ? '#a0b8c9' : '#557a95',
          warning: '#ffc51c',
          positive: '#73be37',
          negative: this.$q.dark.isActive ? '#f78d7f' : '#f35843',
          dark: '',
        }
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.element('NgApp').attrs({
          title: 'ProMoS/NG',
		  version: this.version,
          menuTitle: this.ctx.i18n.i18n.t('menu'),
          colors: this.getColors(),
          disableHeaderInvalidUser: true,
          disableDrawer: !!this.$route.query.headless,
          disableHeader: !!this.$route.query.headless,          
          loading: this.loading,
        }).append('drawer', () => [
          components.spacing(['gutter-y-sm', 'mt-sm']).append(() => [
            components.list('').attrs({ dense: true }).append(() => [
                components.renderIf(this.hasHelp, components.item('Help').attrs({ 
                  clickable: true, 
                  icon: 'help',
                  dense: true,
                }).on({ 
                  onClick: () => this.runHelp(),
                })),
               ...this.menus.map((menu) => components.item(menu.title).attrs({ 
                 clickable: true,  
                 dense: true, 
                 id: menu.title,
                 permissions: menu.permissions,
                 active: (this.ctx.routing.route.path.startsWith(menu.to) && menu.to !== '/') || (menu.to === this.ctx.routing.route.path),
               }).on({ 
                 onClick: () => {
                 if (menu.type === 'a') {
                   this.ctx.routing.openURL(menu.to)
                 } else {
                   this.ctx.routing.push(menu.to)
                 }
               }})),
            ]),     
          ]),
          components.dialog('help', this.dialog.help, (val) => this.dialog.help = val).append(() => [
            components.list().append(() => this.helpList.list.map((item) => components.item(item.label, item.caption).on({
              onClick: () => this.loadHelpFromList(item),
            })))
          ])
        ])
      ]
    },
  }
})