// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    mixins: [
      window.processManager.mixin(),
    ],
    
    // Listen on mounted event...
    beforeCreate() {
      window.components.setParent(this)
      window.processManager.setParent(this)
    },
	
	created() {
		this.ctx.workspace.toService([this.processManagerUuid], true, false)
		this.ctx.endpoints.toService([this.processManagerUuid], true, false)
	},

    mounted() {
      const data = this.getLastZoom()
      if (data) {
        this.zoom = data.zoom
      }
    },

    unmounted() {
      this.ctx.httpClient.delete(this.ctx.endpoints.getHttp(`/discovery/${this.processUuid}`))
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        version: '0.0.1',
        processManagerUuid: this.$route.query.service,
        workspaceUuid: this.$route.query.workspace,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        prevLoadedProcessUuid: '',
        zoom: 0.7,
        dialog: {
          open: false
        },
        endpoint: '',
        name: 'pWebAccess',
      }
    },

    // Define methods for the ui...
    methods: {
      getFileMenu() {
        return [
          {
            id: 'open',
            label: 'open',
            separator: true,
            disable: !!window.components.router().query.instance,
            click: () => this.dialog.open = true,
          },
          {
            id: 'newTab',
            label: 'newTab',
            separator: true,
            disable: !this.endpoint,
            click: () => this.ctx.routing.openURL(this.endpoint),
          }, 
        ]
      },

      getViewMenu() {
        return [
          {
            id: 'zoomIn',
            icon: 'zoom_in',
            label: 'zoomIn',
            click: () => this.zoomState('in'),
          },
          {
            id: 'zoomOut',
            icon: 'zoom_out',
            label: 'zoomOut',
            click: () => this.zoomState('out'),
          }, 
        ]
      },

      setLastInstance(processManagerUuid, instanceUuid, processUuid) {
        this.ctx.rest.lsSetJSON(`pwa:${this.workspaceUuid}:lastInstance`, { processManagerUuid, instanceUuid, processUuid })
      },

      getLastInstance() {
        return this.ctx.rest.lsGetJSON(`pwa:${this.workspaceUuid}:lastInstance`)
      },

      setLastZoom(zoom) {
        this.ctx.rest.lsSetJSON(`pwa:${this.workspaceUuid}:lastZoom`, { zoom })
      },

      getLastZoom() {
        return this.ctx.rest.lsGetJSON(`pwa:${this.workspaceUuid}:lastZoom`)
      },

      // functions
      getBreadcrumb() {
        if (this.workspaceUuid) {
          return window.processManager.getPMBreadcrumbs([
            ...window.processManager.getPMWorkspaceBreadcrumb(),
            {
              label: 'pWebAccess'
            },
          ])
        }

        return window.processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'module')
      },

      async init() {
        this.endpoint = ''
  
        if (this.workspaceUuid) {
          await this.ctx.workspace.fetchWorkspace(this.workspaceUuid)
          const instance = this.getLastInstance() 
          if (instance) {
            this.processUuid = instance.processUuid
            this.instanceUuid = instance.instanceUuid
            this.processManagerUuid = instance.processManagerUuid
          }
        }

        if (this.instanceUuid) {
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
        }
        
        if (this.instanceUuid && this.processUuid) {
          const server = await this.ctx.processManager.fetchServer(this.processManagerUuid, this.instanceUuid, this.processUuid)
          const endpoint = this.ctx.processManager.getEndpoint(server)
          this.loadPromos({
            processUuid: this.processUuid,
            instanceUuid: this.instanceUuid,
            processManagerUuid: this.processManagerUuid,
            endpoint,
            name: this.instanceSettings.name,
          })
        }
      },

      zoomState(state = 'in') {
        if (state === 'out') {
          if (this.zoom > 0.5) {
            this.zoom *= 0.8
          }
        } else {
          if (this.zoom < 10) {
            this.zoom *= 1.2
          }
        }

        this.setLastZoom(this.zoom)
      },

      loadPromos(obj) {
        this.endpoint = ''
		const prev = this.getLastInstance()
        if (prev) {
          this.ctx.httpClient.delete(this.ctx.endpoints.appendService(prev.processManagerUuid).getHttp(`/discovery/${prev.processUuid}`))
        }

        this.ctx.httpClient.post(this.ctx.endpoints.appendService(obj.processManagerUuid).getHttp('/discovery'), {
          name: obj.name,
          slug: obj.processUuid,
          url: obj.endpoint,
          secure: true,
          type: 'pwa',
          redirect: true
        })
        .then(() => {
			
          this.endpoint = this.ctx.endpoints.appendService(obj.processManagerUuid).getHttp(`/service/${obj.processUuid}/`)
          this.name = obj.name  
          this.setLastInstance(obj.processManagerUuid, obj.instanceUuid, obj.processUuid)
        })
      },
    },

    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        window.components.page(this.ctx.globals.getPermissions(['workspacesRead', 'processesRead']), () => this.init()).append(() => [
          window.components.breadcrumbs(this.getBreadcrumb()),
          
          window.components.bar(this.name).createMenu('file', this.getFileMenu()).createMenu('view', this.getViewMenu()),
          
          window.components.section().append(() => [

            window.components.renderIf(!this.endpoint, window.components.card().append(() => [
              window.components.section().append(() => [
                window.components.text('noProjectSelected').attrs(),
              ]),
              window.components.section().append(() => [
                window.components.button('open').on({ onClick: () => this.dialog.open = true }),
              ]),
            ])),
            // position:relative;padding-top:56.25%;
            window.components.renderIf(this.endpoint, window.components.element('div').attrs({ style: { height: '80vh'}}).append(() => [
              // position:absolute;top:0;left:0;width:100%;height:100%;
              window.components.element('div').attrs({ style: { position: 'relative', top: '0', height: '100vh', width: '100%'}, class: 'q-mt-md'}).append(() => [
                // <embed type="text/html" src="snippet.html" width="500" height="200">
                window.components.element('embed').attrs({
                 src: this.endpoint,
                 type: 'text/html',
                 width: '100%',
                 height: '100%',
                 style: {
                   position: 'absolute',
                   top: 0,
                   left: 0,
                   width: '100%',
                   height: '100%',
                   'transform': `scale(${this.zoom})`,
                   'transform-origin': '0 0',
                 }
              }).append(() => [
                  window.components.text('noData'),
              ])              
            ]),

            ])),

            // Dialogs
            // File
            window.components.formDialog('open', this.dialog.open, (val) => this.dialog.open = val, 'public/forms/select-endpoint.js').formOn({
              'onClick:endpoint': (obj) => {
                this.dialog.open = false
                this.loadPromos(obj)
              },
              onCancel: () => this.dialog.open = false,
            }).formAttrs({
              workspaceUuid: this.workspaceUuid, 
              name: 'pWebAccess', 
            }),
          ]),
        ]),
      ]
    },
  }
})