// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    
    mixins: [
      processManager.mixin(),
    ],
    
    // Listen on mounted event...
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        workspaceUuid: this.$route.query.workspace,
        processManagerUuid: this.$route.query.service,
        language: 'de_DE',
        dialog: {
          settings: false,
        },
        form: {
          alarm: {
            format: '',
          },
        }
      }
    },

    // Define methods for the ui...
    methods: {
      // functions
      getBreadcrumb() {
        if (this.workspaceUuid) {
          return processManager.getPMBreadcrumbs([
            ...processManager.getPMWorkspaceBreadcrumb(),
            {
              label: this.$t('alarms')
            },
          ])
        }
      },

      init() {
        this.language = this.ctx.i18n.getLocale().replace('-', '_')
        this.ctx.workspace.toService([this.processManagerUuid]).fetchWorkspace(this.workspaceUuid)
          .then((workspace) => {
            this.workspace = workspace
            if (!this.workspace.alarm) {
              this.workspace.alarm = {}
            }

            if (!this.workspace.alarm.format) {
              this.workspace.alarm.format = ''
            }
            
            this.form = JSON.parse(JSON.stringify(this.workspace))
          })
      },

      getEditMenu() {
        return [
          {
            id: 'settings',
            label: 'settings',
            click: () => this.dialog.settings = true,
            permissions: this.ctx.globals.getPermissions(['workspaceCreate', 'workspaceUpdate']),
          },
        ]
      },

      saveSettings() {
        if (!this.workspace) {
          return
        }

        if (!this.workspace.alarm) {
          this.workspace.alarm = {}
        }

        this.workspace.alarm = { ...this.workspace.alarm, ...this.form.alarm }
        this.workspaceUuid = ''
        this.ctx.workspace.toService([this.processManagerUuid]).postWorkspace(this.workspace)
          .then(() => {
            this.ctx.notify.notifySaved()
          })
        .catch((err) => {
          this.ctx.notify.notifyApiError(err)
        })
        .finally(() => {
          this.dialog.settings = false
          this.workspaceUuid = this.form.uuid
        })
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page(this.ctx.globals.getPermissions(['alarmRead']), () => this.init()).append(() => [
          components.breadcrumbs(this.getBreadcrumb()),
          components.bar('alarms').createMenu('file', this.getEditMenu()),
          components.section().append(() => [
            components.renderIf(this.workspace.uuid, components.element('NgDMSWorkspaceAlarmList').attrs({
              workspaceUuid: this.workspaceUuid,
              serviceUuids: [this.processManagerUuid],
              workspaceConfig: this.workspace,
              maxHeight: this.workspace && this.workspace.instances.length > 1 ? '64vh' : '70vh',
              alarms: [
                {
                  group: 'Alarm',
                  filter: {
                    path: '',
                  },
                  local: this.language,
                  format:  this.workspace && this.workspace.alarm ? this.workspace.alarm.format : '',
                },
              ]
            })),

            components.dialog('', this.dialog.settings, (val) => this.dialog.settings = val).append(() => [
              components.form('settings').append(() => [
                components.input('format', 'text', this.form.alarm.format, (val) => this.form.alarm.format = val).attrs({ readonly: false }),
              ]).on({
                onSubmit: () => this.saveSettings(),
              })
            ]),
          ])
        ])
      ]
    },
  }
})