// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    
    mixins: [
      processManager.mixin(),
    ],
    
    // Listen on mounted event...
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    created() {
      this.ctx.workspace.toService([this.processManagerUuid], true, false)
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        workspaceUuid: this.$route.query.workspace,
        processManagerUuid: this.$route.query.service,
        language: 'de_DE',
      }
    },

    // Define methods for the ui...
    methods: {
      // functions
      getBreadcrumb() {
        if (this.workspaceUuid) {
          return processManager.getPMBreadcrumbs([
            ...processManager.getPMWorkspaceBreadcrumb(),
            {
              label: this.$t('diagrams')
            },
          ])
        }
      },

      init() {
        this.language = this.ctx.i18n.getLocale().replace('-', '_')
        this.ctx.workspace.fetchWorkspace(this.workspaceUuid)
          .then((workspace) => {
            this.workspace = workspace
          })
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page(this.ctx.globals.getPermissions(['diagramsUpdate']), () => this.init()).append(() => [
          components.breadcrumbs(this.getBreadcrumb()),
          components.renderIf(this.workspaceUuid, components.element('NgDiagram').attrs({
            workspaceUuid: this.workspaceUuid,
            serviceUuids: [this.processManagerUuid],
          })),
        ])
      ]
    },
  }
})