// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processName: this.$route.query.name,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        config: null,
        loading: true,
      }
    },

    // Define methods for the ui...
    methods: {
      fetchConfig(pathname) {
        return new Promise((resolve, reject) => {
          this.ctx.filedb.read(pathname, true)
          .then((res) => {
            this.config = res.data.data
            resolve(this.config)
          })
          .catch((err) => {
            reject(err)
          })
        })
      },

      // functions
      async init() {
        this.loading = true
        try {
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
        } catch (e) {
          this.ctx.notify.notifyApiError(e)          
        }

        this.fetchConfig(`/cfg/configurations/${this.processName}.yaml`)
          .then(() => {
            return this.config
          })
          .catch(() => {
            return this.fetchConfig('cfg/configurations/templates/settingsUnknown.yaml')
          })
          .finally(() => {
            this.loading = false
          })
      },
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page(this.ctx.globals.getPermissions(['processesUpdate']), () => this.init()).append(() => [
          
          components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'settings')),
          
          components.bar('process'),
          
          components.section().attrs({ class: 'row justify-center' }).append(() => [
            components.card().attrs({ loading: this.loading, class: 'col-12 col-md-6' }).append(() => [
              components.section().attrs({ class: 'full-width' }).append(() => [
                components.renderIf(!this.loading, components.processConfig(
                  undefined, 
                  this.ctx.endpoints.getLocalService(), 
                  this.instanceUuid, 
                  this.processUuid, 
                  this.config,
                ).attrs({
                  serviceUuids: [this.processManagerUuid],
                })),
              ]),
            ]),
          ]),
        ]),
      ]
    },
  }
})