// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    created() {
      // logic
      this.ctx.endpoints.toService([this.processManagerUuid], true, false)
      this.processName = this.$attrs.name || this.$route.query.name
      this.instanceUuid = this.$attrs.instance || this.$route.query.instance
      this.processUuid = this.$attrs.process || this.$route.query.process
      this.history = this.$attrs.statusIds || this.$route.query.statusIds || ['']
      this.isUnmounted = false
      this.request.statusId = Array.isArray(this.history) && this.history.length ? this.history[this.history.length - 1] : ''
    },

    // Listen on unmounted event...
    unmounted() {
      // logic
      clearTimeout(this.reconnectId)
      clearTimeout(this.reconnectLogId)
      Object.keys(this.reconnectLogIds).forEach((logId) => clearTimeout(this.reconnectLogIds[logId]))
      Object.keys(this.notificationIds).forEach((id) => clearTimeout(this.notificationIds[id]))
      
      if (this.socket && this.socket.close) {
        this.socket.close(1000)
      }

      if (this.logSocket && this.logSocket.close) {
        this.logSocket.close(1000)
      }

      Object.values(this.logSockets).forEach((sock) => {
        if (sock && sock.close) {
          sock.close(1000)
        }
      })
      
      this.isUnmounted = true
      this.ctx.httpClient.delete(this.ctx.endpoints.getHttp(`/discovery/${this.processUuid}`))
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        isUnmounted: false,
        reconnectId: 0,
        reconnectLogId: 0,
        reconnectLogIds: {},
        notificationIds: {},
        processName: this.$route.query.name,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        endpoint: '',
        isConnected: true,
        loading: true,
        connecting: true,
        isStatic: true,
        socket: {},
        logSocket: {},
        logSockets: {},
        selects: {},
        response: {},
        columns: [],
        rows: [],
        rowsAsObject: {},
        pagination: components.createDefaultPagination(this.$attrs.pagination || 100),
        maxLength: 1000,
        maxLengthTotal: 100000,
        enabled: true,
        history: this.$route.query.statusIds || [''],
        request: {
          command: 'subscribe',
          statusId: ''
        },
        dialog: {
          log: false,
        },
        isinit: false,
        hasModule: false,
        lastStatusId: 0,
        statusTabItem: {
          name: 'status',
          label: 'status',
          icon: 'checklist',
        },
        tabs: [
        ],
        tab: 'status',
        enumLogs: [],
        columnsLog: {},
        rowsLog: {},
        paginationLog: components.createDefaultPagination(100),
        panel: [],
        tabNotifications: {},
        selectedLog: {},
      }
    },

    watch: {
      enumLogs: {
        deep: true,
        handler(newEnumLogs) {
          const tabs = [this.statusTabItem]
          tabs.push(...newEnumLogs.map((enumLog) => ({
            name: enumLog.id,
            label: enumLog.label,
            icon: 'description',
            options: enumLog
          })))

          this.tabs = tabs
          newEnumLogs.forEach((enumLog) => {
            this.initLogSocketPerLogId(enumLog.id)
          })
        },
      },
      tab() {
        this.setTabPanel(this.tab)
      }
    },
  
      
    // Define methods for the ui...
    methods: {
      // functions 
      async init() {
        if (this.isinit) {
          return
        }
        this.isinit = true
        try {
          this.loading = true
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.processServer = await this.ctx.processManager.fetchServer(this.processManagerUuid, this.instanceUuid, this.processUuid)
          let host = this.ctx.processManager.getHost(this.processServer)
          // If we need to deal with dynamic port, read it from discovery db
          if (this.processServer.location.port <= 0) {
            const discoveryItem = await this.ctx.httpClient.get(this.ctx.endpoints.getHttp(`/discovery/${this.processServer.name}`))
            host = discoveryItem.data.options.url
          }

          this.endpoint = this.ctx.routing.getHttpUrl(host) 
          this.hasModule = window.processManager.hasModule(this.processSettings.name)

          await this.ctx.httpClient.post(this.ctx.endpoints.getHttp('/discovery'), {
            secure: true,
            url: this.endpoint,
            type: 'process',
            name: this.processUuid,
            slug: this.processUuid,
            isLocal: true,
          })
          
          this.initSocket()
          this.initLogSocket()
          this.loading = false
        } catch (e) {
          this.ctx.notify.notifyApiError(e)
          this.loading = false
        }

      },
  
      getRootElement() {
    
        if (!this.$attrs.process) {
          return components.page(this.ctx.globals.getPermissions(['processesUpdate']), () => this.init())
        }

        if (this.$attrs.status === undefined || this.$attrs.status !== 0) {
          this.init()
        }
        
        return components.element('NgSection').attrs({ permissions: this.ctx.globals.getPermissions(['processesUpdate']) })
      },

      sendSubscribe() {
        this.loading = true
        this.socket.send(this.request)
      },

      sendUnsubscribe(statusId) {
        this.socket.send({ command: 'unsubscribe', statusId: statusId })
      },

      sendAction(actionId) {
        this.socket.send({ command: 'action', statusId: null, actionId })
        components.notify().positive(actionId)
      },

      sendSubState(statusId) {
        // TODO: We need to close the connection because unsubscribe not worked.
        if (this.socket) {
          this.socket.close(1000)
        }
        this.rowsAsObject = {}
        this.rows = []
        this.columns = []
        this.history.push(statusId)
        if (!this.$attrs.process) {
          components.router().replace({ statusIds: this.history })
        }
        // Assign new statusId to request to send it on connected
        this.request.statusId = statusId
        setTimeout(() => {
          this.initSocket()
        }, 500)
      },

      sendBackSubState() {
        // TODO: We need to close the connection because unsubscribe not worked.
        if (this.socket) {
          this.socket.close(1000)
        }
        this.rowsAsObject = {}
        this.rows = []
        this.columns = []
        this.history.pop()
        if (!this.$attrs.process) {
          components.router().replace({ statusIds: this.history })
        }
        // Assign new statusId to request to send it on connected
        this.request.statusId = this.history[this.history.length - 1]
        this.initSocket()
      },

      getHistoryLabel() {
        return this.history[this.history.length - 1] || 'status'
      },

      getHistoryBackLabel() {
        return (this.history.length >= 2 ? this.history[this.history.length - 2] : '') || 'status'
      },

      getColIdent(index) {
        return `col-${index}`
      },

      handelColumns(event) {
        this.isStatic = event.isStatic
        if (this.isStatic) {
          this.enabled = true
        }
        if (event.columnHeaders) {
          this.columns = event.columnHeaders.map((col) => {
            const ident = this.getColIdent(col.index)
            return {
              i18nLabel: col.title,
              name: ident,
              field: (val) => val && val[ident] ? val[ident].content : '',
              align: 'left',
              sortable: true,
            }
          })
        }
      },

      handleRows(event) {
        if (event.rows) {
          event.rows.forEach((row) => {
            if (!this.rowsAsObject[row.rowId]) {
              this.rowsAsObject[row.rowId] = {}
            }
            
            row.cells.forEach((cell) => {
              this.rowsAsObject[row.rowId][this.getColIdent(cell.columnIndex)] = cell
            })
          })

          if (this.isStatic) {
            this.rows = Object.assign(this.rows, Object.values(this.rowsAsObject));
          } else {
            if (this.rows.length > this.maxLength - 1) {
              this.rows.pop()
            }
            this.rows.unshift(Object.values(JSON.parse(JSON.stringify(this.rowsAsObject)))[0])
            
          }
        }
      },

      eventHandler(event) {
          switch (event.event) {
            case 'init':
              this.loading = false
              this.handelColumns(event)
              // render panel
              if (this.tab === 'status') {
                this.setTabPanel(this.tab)
              }
              break;
            case 'rows':
              if (this.enabled) {
                this.handleRows(event)    
              }
              break;
			case 'resetTable':
			  this.rows.length = 0;
			  this.rowsAsObject = {};
			  break;
            default:
              break;
          }
      },
      
      messageHandler(data) {
        if (data.events) {
          data.events.forEach((event) => this.eventHandler(event))
        }
      },
      
      initSocket() {
        if (this.isUnmounted || this.processStatus.status === 0) {
          return
        }
        const { Socket } = this.ctx
        this.connecting = true
        clearTimeout(this.reconnectId)
        this.socket = new Socket()
        this.socket.setAuth(true)
        this.socket.onConnected(() => {
          this.isConnected = true
          this.connecting = false
          this.sendSubscribe()
        })

        this.socket.onClosed((ev) => {
          this.connecting = false
          if (ev.code !== 1000) {
            this.isConnected = false
            this.reconnectId = setTimeout(() => this.initSocket(), 5000)
          }
        })

        this.socket.onError((err) => {
          this.isConnected = false
          this.loading = false
          this.ctx.notify.plugin.error(err.message)
          if (this.panel.length <= 0 && this.tab === 'status') {
            this.setStatusPanel()
          }
        })

        this.socket.onMessage((data = { type: String, data: Object}) => {
          this.isConnected = true
          try {
            this.messageHandler(data.data)
          } catch (e) {
            console.error(e)
          }
        })
        this.socket.connect(this.ctx.endpoints.getWs(`/service/${this.processUuid}/v1/status`))
      },

      initLogSocket() {
        if (this.isUnmounted) {
          return
        }
        const { Socket } = this.ctx
        clearTimeout(this.reconnectLogId)
        this.logSocket = new Socket()
        this.logSocket.setAuth(true)
        this.logSocket.onConnected(() => {
          this.logSocket.send({command:'enumLogs'})
        })

        this.logSocket.onClosed((ev) => {
          if (ev.code !== 1000 && !this.isUnmounted) {
            this.reconnectLogId = setTimeout(() => this.initLogSocket(), 1000)
          }
        })

        this.logSocket.onError((err) => {
          this.ctx.notify.plugin.error(err.message)
        })

        this.logSocket.onMessage((data = { type: String, data: Object}) => {
          try {
            this.logMessageHandler(data.data)
          } catch (e) {
            console.error(e)
          }
        })

        this.logSocket.connect(this.ctx.endpoints.getWs(`/service/${this.ctx.endpoints.getLocalService()}/v1/logs/${this.processUuid}`))
      },

      initLogSocketPerLogId(logId, force) {
        if (this.logSockets[logId] && !force) {
          return
        }

        const id = logId

        if (this.isUnmounted) {
          return
        }
        
        const { Socket } = this.ctx
        clearTimeout(this.reconnectLogIds[id])
        const sock = new Socket()
        sock.setAuth(true)

        sock.onConnected(() => {
          sock.send({command:'enumLogs'})
        })

        this.rowsLog[logId] = []
        this.columnsLog[logId] = []
        
        sock.onClosed((ev) => {
          if (ev.code !== 1000 && !this.isUnmounted) {
            this.reconnectLogIds[id] = setTimeout(() => this.initLogSocketPerLogId(id, true), 5000)
          }
        })

        sock.onError((err) => {
          this.ctx.notify.plugin.error(err.message)
          this.tabNotifications[logId] = 'X'
        })

        sock.onMessage((data = { type: String, data: Object}) => {
          try {
            if (data.data.event === 'enumLogs') {
              sock.send({command:'subscribeLog', logId, maxLines: this.maxLength})
            }
            this.logMessageHandler(data.data, id)
          } catch (e) {
            console.error(e)
          }
        })

        sock.connect(this.ctx.endpoints.getWs(`/service/${this.ctx.endpoints.getLocalService()}/v1/logs/${this.processUuid}`))
        this.logSockets[logId] = sock
      },

      logMessageHandler(data, logId) {
        let newLogCount = 0
        switch (data.event) {
          case 'enumLogs':
            // only trigger enumLogs when general socket read not logId specific socket
            if (logId === undefined) {
              this.enumLogs = data.logs.map((log, idx) => ({ id: idx + 1, label: log })) 
            }
            break
          case 'logLines':
            newLogCount = data.lines.length
            if (this.columnsLog[logId] && this.columnsLog[logId].length <= 0 && newLogCount > 0) {
              this.logHandleColumns(data.lines, logId)
            }
            
            this.logHandleRows(data.lines, logId)
            break
          case 'error':
            this.ctx.notify.plugin.error(data.message)
            break
        }

        if (newLogCount > 0) {
          clearTimeout(this.notificationIds[logId])
          this.tabNotifications[logId] = `${newLogCount}`
          this.notificationIds[logId] = setTimeout(() => {
            this.tabNotifications[logId] = '✓'
          }, 5000)
        }
      },

      logHandleColumns(lines = [], logId) {
        if (!this.columnsLog[logId]) {
          return
        }
        let colsByName = {}
        lines.forEach((line) => {
          const cols = components.createTableColsByObject(line)
          cols.forEach((col) => {
            col.style = 'max-width: 250px;'
            col.classes = 'text-ellipsis ellipsis'
            colsByName[col.name] = col;
          })
        })

        this.columnsLog[logId].push(...Object.values(colsByName));
      },

      logHandleRows(lines = [], logId) {
        if (!this.rowsLog[logId]) {
          return
        }
        lines = lines.filter((log) => log.msg !== '').reverse()
        this.rowsLog[logId].unshift(...lines)
        if (this.rowsLog[logId].length > this.maxLengthTotal) {
          this.rowsLog[logId].splice(this.maxLengthTotal)
        }
      },

      logClickRow(logItem) {
        this.selectedLog = logItem
        this.dialog.log = true
      },

      getLogDialogValueElement(value) {
        let val = value
        let component = components.element('div').attrs({ style: 'overflow: auto;' }).append(() => [
          val,
        ])
        
        if (typeof value === 'string' && value.startsWith('{')) {
          try {
            val = JSON.stringify(JSON.parse(value), null, 2)
            component = components.element('NgFormInput').attrs({ type: 'editor', lang: 'json', modelValue: val })
          } catch (e) {
            console.error(e)
          }
        }

        return component
      },

      logGetBadgeColor(level) {
        let color = 'grey'
        
        switch (level) {
          case 'error':
            color = 'red'
            break
          case 'info':
            color = 'blue-4'
            break
          case 'debug':
            color = 'dark'
            break
          case 'warn':
            color = 'orange-4'
            break
          case 'panic':
            color = 'negative'
            break
        }
  
        return color
      },
      
      getActionMenu(row, cell) {
        return components.menu().attrs({ 
          items: this.getActionMenuItems(cell.options), 
          enableGlobalClick: true,
        }).on({ 
          'onClick:menu': (ident) => {
            switch(cell.format) {
              case 'actions':
                this.sendAction(ident)
                break;
              case 'subStates':
                this.sendSubState(ident)
                break
              default:
                components.notify()
                break
            }
          },
        })
      },
      
      getActionMenuItems(options) {
        const opts = options.map((opt) => ({
          label: opt.text,
          id: opt.id,
        }))
        return opts
      },

      getStatusColor(cell) {
        switch (cell.status) {
          case 'none':
            return 'grey-5'
          case 'ok':
            return 'positive'
          case 'error':
            return 'negative'
          case 'warning':
            return 'orange'
          default:
            return ''
        }
      },

      getStatusIcon(cell) {
        switch (cell.status) {
          case 'none':
            return 'label'
          case 'ok':
            return 'done'
          case 'error':
            return 'notifications'
          case 'warning':
            return 'warning'
          default:
            return ''
        }
      },
      
      getCellComponent(row, cell, type = 'row') {
        let output = ''
        
        switch (cell.format) {
          case 'subStates':
          case 'actions':
            output = this.getActionMenu(row, cell)
            if (type === 'item') {
              output.attrs({
                label: 'show',
                color: this.$attrs.grid ? 'primary' : undefined,
                flat: false,
              })
            }
            break
          case 'bool':
            if (cell.status !== 'none') {
              output = components.badge('', cell.content ? 'done' : 'block', this.getStatusColor(cell))
            } else {
              output = components.badge('', cell.content ? 'done' : 'block', cell.content ? 'positive' : 'grey')
            }
            break;
          case 'numeric':
            if (cell.status !== 'none' && cell.content !== undefined && cell.content !== null) {
              output = components.badge(cell.content, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else {
              output = cell.content
            }
            break;
          case 'timestamp':
            if (cell.status !== 'none' && cell.content !== undefined && cell.content !== null) {
              output = components.badge(cell.content, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else {
              output = new Date(cell.content).toLocaleString()
            }
            break;
          default:
            output = typeof cell.content === 'string' ? this.ctx.i18n.mdt(cell.content) : cell.content;
            if (cell.status !== 'none' && output) {
              output = components.badge(output, this.getStatusIcon(cell), this.getStatusColor(cell))
            } else if(cell.content.includes('\n')) {
              output = components.element('pre').append(() => [ output ])
            }
        }
        return output
      },

      setStatusPanel() {
        this.panel = [
          components.renderIf(!this.isConnected, components.bannerNegative(this.ctx.i18n.i18n.t('notConnected'))),
          components.renderIf(this.processStatus.status === 0 || this.$attrs.status === 0, components.bannerWarn(this.ctx.i18n.i18n.t('enableTheProcess'))),
          components.renderIf(this.processStatus.status !== 0 && this.$attrs.status !== 0, components.table(this.lastStatusId, this.columns, this.rows)
            .pagination(this.pagination, (val) => this.pagination = val)
            .attrs({
              maxHeight: this.$attrs.maxHeight || '75vh',
              clickable: false,
              flat: !!this.$attrs.flat,
              grid: !!this.$attrs.grid,
              loading: (this.loading || this.connecting) && this.processStatus.status !== 0,
            })
            .append('top-left', () => {
              const render = [
                components.renderIf(
                  this.history.length >= 2,
                  components.button(this.getHistoryBackLabel(), 'chevron_left').on({
                    onClick: () => {
                      this.sendBackSubState()
                    }
                  })
                )
              ]
              
              if (!this.isStatic && !this.$attrs.process) {
                render.push(components.button('start', 'play_arrow').attrs({
                  disable: this.enabled,
                  color: 'positive',
                }).on({
                  onClick: () => this.enabled = !this.enabled,
                }),
                components.button('stop', 'stop').attrs({
                  color: 'negative',
                  disable: !this.enabled,
                }).on({
                  onClick: () => this.enabled = !this.enabled,
                }))
              }
              
              return [
                components.flex().append(() => render)
              ]
            })
            .append('body-cell', (props) => {
              const item = props.row[props.col.name] !== undefined ? props.row[props.col.name] : { content: '' }
              return [
                components.td(props).attrs({
                  style: 'min-width: 150px; max-width: 850px;',
                  class: 'text-overflow',
                }).append(() => [
                  this.getCellComponent(props.row, item),
                ])
              ]
            }).append('item', (props) => [
              components.tdItem(components.card().attrs({ flat: false }).append(() => [
                components.list().append(() => [
                  ...this.columns.map((col) => {
                    const item = props.row[col.name] !== undefined ? props.row[col.name] : { content: '' }
                    return components.item(col.i18nLabel).append('subtitle', () => [
                      this.getCellComponent(props.row, item, 'item'),
                    ])
                  })
                ])
              ]))
          ])),
        ]
      },

      setLogPanel(logIdx = 1) {
        this.panel = [
          components.table('', this.columnsLog[logIdx], this.rowsLog[logIdx])
            .pagination(this.paginationLog, (val) => this.paginationLog = val)
            .on({
              'onClick:row': (props) => this.logClickRow(props.row, logIdx),
            })
            .attrs({ 
              configIdent: 'pm-log',
              maxHeight: this.$attrs.maxHeight || '75vh',
              clickable: true,
              flat: !!this.$attrs.flat,
              grid: !!this.$attrs.grid,
            }).append('body-cell-msg', (props) => [
            components.td(props).append(() => [
              props.value
            ]).attrs({ 
              class: 'ellipsis', 
              style: { 
                maxWidth: this.ctx.platform.mobile ? '250px' : '500px' 
              } 
            })
          ]).append('body-cell-timestamp', (props) => [
            components.td(props).append(() => [
              new Date(props.value).toLocaleString()
            ])
          ]).append('body-cell-level', (props) => [
            components.td(props).append(() => [
              components.badge().attrs({ color: this.logGetBadgeColor(props.value) }).append(() => [
                props.value,
              ])
            ])
          ])
        ]
      },

      setTabPanel(tab) {
        switch (tab) {
          case 'status':
            this.setStatusPanel()
            break
          default:
            this.setLogPanel(tab)
            break
        }
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        this.getRootElement().append(() => [
            components.renderIf(!this.$attrs.process,
               components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'status'))
            ),
            
            components.renderIf(!this.$attrs.process, 
              components.bar('process').createMenu('file', [
                {
                  id: 'player',
                  label: this.enabled ? 'stop' : 'start',
                  icon: this.enabled ? 'stop' : 'play_arrow',
                  disable: this.isStatic,
                  click: () => this.enabled = !this.enabled,
                }
              ]).createMenu('view', [
                {
                  id: 'logs',
                  label: 'logs',
                  icon: 'description',
                  permissions: this.ctx.globals.getPermissions(['changelogRead']),
                  click: () => {
                    components.router().push('/process-manager/instances/processes/logs/index', { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
  
                {
                  id: 'status',
                  label: 'status',
                  icon: 'checklist',
                  disable: true,
                  active: true,
                  click: () => {
                    components.router().push('/process-manager/instances/processes/status/index', { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
  
                {
                  id: 'module',
                  label: 'module',
                  icon: 'view_module',
                  disable: !this.hasModule,
                  click: () => {
                    components.router().push(`/modules/${this.processSettings.name}/index`, { 
                      service: this.$route.query.service,
                      instance: this.instanceUuid,
                      process: this.processUuid,
                      processName: this.$route.query.name,
                    })
                  }
                },
            ])
            ),

            components.element(this.$attrs.dense ? 'div' : 'NgCardSection').attrs({ class: 'q-gutter-md' }).append(() => [
              // Tabs
              components.element('NgTabs').attrs({
                modelValue: this.tab,
                'onUpdate:modelValue': (val) => this.tab = val,
                tabs: this.tabs,
                disableQueryTab: true,
                disableTitle: true,
                dense: true,
                flat: true,
                notifications: this.tabNotifications,
              }).append(this.tab, () => this.panel),

              // Log dialog
              components.dialog('log', this.dialog.log, (val) => this.dialog.log = val).append(() => [
              components.list().append(() => [
                components.item('level', this.selectedLog.level),
                components.item('timestamp', new Date(this.selectedLog.timestamp).toLocaleString()),
              ]),
  
              components.section().append(() => [
                components.list().attrs({ bordered: true }).append(() => [
                  ...Object.keys(this.selectedLog).map((log) => components.item(log).append('subtitle', () => [
                     this.getLogDialogValueElement(this.selectedLog[log]),
                  ]))
                ]),
              ]),
            ]),  
          ]),     
        ]),
      ]
    },
  }
})