import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
    },
    
    // Listen on unmounted event...
    unmounted() {
      // logic
    },
      
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        workspaceUuid: this.$route.query.workspace,
        processManagerUuid: this.$route.query.service,
        dialog: {
          add: false,
          remove: false,
		  exportWorkspace: false,
        },
        loading: false,
        apps: [],
        modules: {},
        moduleNamesUsedInWorkspace: [],
        workspace: {},
        instances: [],
      }
    },

    methods: {
      async init() {
        this.moduleNamesUsedInWorkspace = []
        this.loading = true
        this.workspace = await this.ctx.workspace.toService([this.processManagerUuid]).fetchWorkspace(this.workspaceUuid)
        this.fetchModules()
        setTimeout(() => {
          this.loading = false
        }, 500)
      },

      fetchInstances() {
        if (this.workspace.promosnts && this.workspace.promosnts.length && !this.moduleNamesUsedInWorkspace.find((item) => item.id === 'DMS')) {
          this.moduleNamesUsedInWorkspace.unshift(this.modules.processes.find((proc) => proc.id === 'DMS'))
        }

        if (!this.workspace.instances || this.workspace.instances.length <= 0) {
          this.instances = []
          this.filterModules()
          return
        } 
        
        this.workspace.instances.forEach((inst) => {
          this.ctx.processManager.toService([this.processManagerUuid]).fetchInstanceSettings(inst.processManagerUuid, inst.instanceUuid)
            .then((instance) => {
              if (!Array.isArray(instance)) {
                  this.instances.push(instance)
              }
              this.filterModules()
            })
          .catch((err) => {
            this.ctx.notify.notifyApiError(err)
            this.loading = false
          })
        })
      },
      
      fetchModules() {
        this.ctx.httpClient.get(this.ctx.endpoints.getHttp('/service/rest/v1/filedb/cfg/modules.json'))
          .then((res) => {
            this.modules = res.data.data.data
            this.apps = this.modules.apps
            this.fetchInstances()
          })
          .catch((err) => {
            components.notify().negative(err)
            this.loading = false
          })
      },

      removeWorkspace() {
        this.ctx.workspace.toService([this.processManagerUuid]).deleteWorkspace(this.workspace.uuid)
          .then(() => {
            this.$router.push('/workspaces')
          })
      },

      filterModules() {
        const modules = {}
        this.instances.forEach((inst) => {
          inst.processes.forEach((proc) => {
            modules[proc.name] = true
          })
        })

        const moduleNames = Object.keys(modules)
        this.moduleNamesUsedInWorkspace = this.modules.processes.filter((proc) => moduleNames.includes(proc.id) || proc.static)

        if (this.instances.length === this.workspace.instances.length) {
          this.loading = false
        }
      },

      getBreadcrumb() {
        return processManager.getPMBreadcrumbs(processManager.getPMWorkspaceBreadcrumb())
      }
      
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page(this.ctx.globals.getPermissions(['workspacesRead']), () => {
          this.init()
        }).append(() => [
          components.breadcrumbs(this.getBreadcrumb()),
          
          components.bar('workspace').createMenu('file', [
            {
              id: 'edit',
              label: 'edit',
              icon: 'edit',
              permissions: this.ctx.globals.getPermissions(['workspacesUpdate']),
              click: () => {
                this.dialog.add = true
              }
            },
            {
              id: 'delete',
              label: 'delete',
              icon: 'delete',
              permissions: this.ctx.globals.getPermissions(['workspacesDelete']),
              click: () => {
                this.dialog.remove = true
              }
            },
			{
              id: 'export',
              label: 'export',
              icon: 'download',
              permissions: this.ctx.globals.getPermissions(['workspacesRead']),
              click: () => {
                this.dialog.exportWorkspace = true
              }
            },
          ]).attrs({
            id: 'workspace-toolbar',
          }),

          window.components.loading(undefined, this.loading, (val) => this.loading = val).attrs({ timeout: 1000}),
          window.components.card().append(() => [
            window.components.section().append(() => [
              window.components.renderIf(!this.loading && this.moduleNamesUsedInWorkspace.length <= 0, components.bannerInfo('noModulesFound')),
              components.renderIf(!this.loading, window.components.getButtonsGrid(this.moduleNamesUsedInWorkspace, {
                workspace: this.workspaceUuid,
                service: this.$route.query.service,
              })),
            ]),
          ]),
		  
		  window.components.dialog().attrs({
              title: 'exportWorkspace',
              seamless: false,
              modelValue: this.dialog.exportWorkspace,
              'onUpdate:modelValue': (val) => this.dialog.exportWorkspace = val,
            }).append(() => [
				window.components.element('NgWorkspaceExport').attrs({
                  workspaceUuid: this.workspaceUuid,
				  serviceUuid: this.$route.query.service,
                })
            ]),
          
          components.dialogDelete(this.workspace.name, this.dialog.remove, (val) => this.dialog.remove = val).on({
            onOk: () => this.removeWorkspace(),
          }), 

          components.formDialog('workspace', this.dialog.add, (val) => this.dialog.add = val, 'public/forms/workspace.js').formOn({
            onSubmit: () => {
              this.dialog.add = false
              this.init()
            },
            onCancel: () => this.dialog.add = false,
          }).formAttrs({
            workspaceUuid: this.workspaceUuid,
            serviceUuid: this.processManagerUuid,
          }),

        ])
      ]
    },
  }
})