import useScripts from '/service/rest/v1/files/data/public/js/useScripts.js'

window.ngScript(() => {
  return {
    name: 'Gradient color',
    description: 'demo1Description',
    properties: ['fill'],
    config() {
      return  {
        startPanel: 'settings',
        panels: [
          {
            name: 'settings',
            header: 'settings',
            items: [
              {
                type: 'navigation',
                props: {
                  to: 'colorStops',
                  label: 'colors',
                }
              },
              {
                type: 'navigation',
                props: {
                  to: 'datapoints',
                  label: 'datapoints',
                }
              },
			  {
                type: 'navigation',
                props: {
                  to: 'values',
                  label: 'values',
                }
              }				  
            ]
          },
          {
            name: 'colorStops',
            header: 'colors',
            items: [
              {
                type: 'list',
                props: {    
                  scope: 'data.computedItem.colors',
                  default: [],
                  to: 'color',
                  avatar: true,
                  label: '{{label}}',
                  addable: {
                    to: 'color',
                    default: {
                      label: '',
                      color: '#ffffff',
                    },
                  },
                  index: 'color',
                  sortable: true,
                  deletable: true,
                }
              },					
            ]
          },
		  {
            name: 'values',
            header: 'values',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data.computedItem',
                  schema: [
                    {
                      type: 'number',
                      scope: 'values.min',
                      label: 'min',
                      col: 'col-4',
                      default: 0,
                    },
                    {
                      type: 'number',
                      scope: 'values.max',
                      label: 'max',
                      col: 'col-4',
                      default: 100,
                    },
					{
                      type: 'checkbox',
                      scope: 'values.changeDirection',
                      label: 'changeDirection',
                      col: 'col-4',
					  default: false,
                    },
                  ]
                }
              },					
            ]
          },
          {
            name: 'color',
            header: 'color',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data.computedItem.colors.{{index.color}}',
                  schema: [
                    {
                      type: 'text',
                      scope: 'label',
                      label: 'label',
                      col: 'col-6',
                      required: true,
                    },
                    {
                      type: 'color',
                      scope: 'color',
                      label: 'color',
                      col: 'col-6',
                      default: 'blue',
                      required: true,
                    }
                  ]
                }
              },					
            ]
          },
          {
            name: 'datapoints',
            header: 'datapoints',
            items: [
              {
                type: 'list',
                props: {    
                  scope: 'data.computedItem.childLinks',
                  default: [],
                  to: 'datapoint',
                  avatar: true,
                  label: '{{link.path || path}}',
                  addable: {
                    to: 'datapoint',
                    default: {},
                  },
                  index: 'datapoint',
                  deletable: true,
                }
              },					
            ]
          },
          {
            name: 'datapoint',
            header: 'datapoint',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data.computedItem',
                  schema: [
                    {
                      type: 'component',
                      element: 'NgComputedLinkChooser',
                      attrs: {
                        flat: false,
                        scope: 'childLinks.{{index.datapoint}}',
                        'scope.workspaceUuid': 'workspaceUuid',
                        'scope.serviceUuids': 'serviceUuids',
                        advanced: true,
                        enableTypes: ['int', 'uint', 'double'],
                        class: 'col-12 col-md-6',
                      }
                    },
                    {
                      type: 'number',
                      scope: 'childLinks.{{index.datapoint}}.options.ratio',
                      label: 'ratio',
                      sideLabel: '%',
                      min: 0,
                      max: 100,
                      class: 'col-12 col-md-6',
                    },
                  ]
                }
              },					
            ]
          },
        ]
      }
    },
			
    render() {
		const scripts = useScripts();
		
        const temperatures = [];
		const ratios = [];
		const colorStops = [];
		
		const children = this.computedObject.getChildren();
		const computedItem = this.computedObject.computedItem;
		const colors = computedItem.colors.map(color => color.color);
		let minTemp = 0;
		let maxTemp = 100;
		let changeDirection = false;
		if (computedItem.values) {
			if (computedItem.values.min) {
				minTemp = computedItem.values.min;
			}
			if (computedItem.values.max) {
				maxTemp = computedItem.values.max;
			}
			if (computedItem.values.changeDirection) {
				changeDirection = computedItem.values.changeDirection;
			}
		}
        
		// get ratios
		if (computedItem) {
			computedItem.childLinks.forEach((childLinks) => {
				if (childLinks.options && childLinks.options.ratio) {
					ratios.push({
						ratio: childLinks.options.ratio,
						link: childLinks.link
					})
				}
			})
		}
		
		// set temperatures object
        children.forEach((child) => { 
            const value = child.getOriginalValue();
			const ratio = ratios.find(ratio => (ratio.link === child.link)); 
			if (ratio) {
				temperatures.push({
					temperature: value,
					ratio: ratio.ratio,
				})
			}
        });
						
		temperatures.forEach(({ temperature, ratio }) => {
		  colorStops.push({
			offset: scripts.calculateOffsetFromRatio(ratio),
			color:  scripts.interpolateTemperatureColor(temperature, colors, minTemp, maxTemp)
		  });
		});
		
		const result = {
		  colorStops: colorStops,
		  coords: {
			x1: 0,
			x2: 0,
			y1: changeDirection ? 0 : 1,
			y2: changeDirection ? 1 : 0
		  },
		  type: 'linear',
		  gradientUnits: 'percentage'
		}
		
		this.computedObject.store.designer.set(this.computedObject.computedObject, this.computedObject.computedProperty, result);
				
      return this.computedObject.computedObject[this.computedObject.computedProperty];
    }
  }
})