// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
	  'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-card-outline',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'feedbackSuffix',
                    scope: 'feedback',
                    col: 'col-12',
                  },
                  
                  {
                    type: 'text',
                    default: 'title',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'color-palette',
                    default: 'green',
                    label: 'trueColor',
                    scope: 'attrs.trueColor',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'color-palette',
                    default: 'grey',
                    label: 'falseColor',
                    scope: 'attrs.falseColor',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'pulse',
                    scope: 'attrs.pulse',
                    col: 'col-6',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'inverse',
                    scope: 'attrs.inverse',
                    col: 'col-6',
                    'call.hidden': 'return !ctx.data.attrs.pulse',
                    sideSlot: 'link',
                  },
                  {
                    type: 'select',
                    default: 'md',
                    label: 'size',
                    scope: 'attrs.size',
                    col: 'col-12',
                    options: [
                      { label: 'xs', value: 'xs' },
                      { label: 'sm', value: 'sm' },
                      { label: 'md', value: 'md' },
                      { label: 'lg', value: 'lg' },
                    ]
                  },
                  {
                    type: 'select',
                    default: 'sm',
                    label: 'padding',
                    scope: 'attrs.padding',
                    col: 'col-12',
                    options: [
                      { label: 'xs', value: 'xs' },
                      { label: 'sm', value: 'sm' },
                      { label: 'md', value: 'md' },
                      { label: 'lg', value: 'lg' },
                    ]
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        value: false,
        feedbackLinks: [],
        label: this.item.attrs.label || 'value',
        feedbackValues: {},
        isProcessing: false,
        processingTimeoutId: 0,
      }
    },

    watch: {
      isProcessing(val) {
        clearTimeout(this.processingTimeoutId)
        if (!val) {
          return
        }
        this.processingTimeoutId = window.setTimeout(() => {
          this.isProcessing = false;
        }, 5000)
      },
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },

    created() {
      this.item.valueType = 'text'
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '', this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
      
      this.feedbackLinks = []
      this.feedbackValues = {}
      if (this.item.feedback) {
        const processUuid = this.computedObject.getProcessUuid()
        const feedbackPathes = this.computedObject.getPathes(this.item.feedback) || []
        feedbackPathes.forEach((feedbackPath) => {
          const link = this.computedObject.getLink(feedbackPath)
          this.panel.setLink(link)
          this.panel.on(processUuid, link.path, this.onFeedbackEvent)
          this.feedbackLinks.push(link);
        })
      }
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '', this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
      if (this.feedbackLinks && this.feedbackLinks.length) {
        const processUuid = this.computedObject.getProcessUuid()
        this.feedbackLinks.forEach((link) => {
          this.panel.off(processUuid, link.path, this.onFeedbackEvent)
        })
      }
    },
	
    methods: {
      hasFeedback(values) {
        return !!Object.values(values).find((val) => !!val)
      },
      onEvent(ev) {
        this.isProcessing = false;
        const value = this.panel.getEventValue(ev)
        this.value = value;
      },
      onFeedbackEvent(ev) {
        this.isProcessing = false;
        const value = this.panel.getEventValue(ev)
        const path = this.panel.getEventPath(ev)
        this.feedbackValues[path] = value;
      },
      emitValue(value) {
        this.isProcessing = true;
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value: value,
            }
          ]
        });
      },
      emitValuePulse() {
        this.isProcessing = true;
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value: !!this.item.attrs.inverse,
            },
            {
              path,
              value: !this.item.attrs.inverse,
            },
            {
              path,
              value: !!this.item.attrs.inverse,
            }
          ]
        });
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgActionButton',
          props: {
            ...(this.item.attrs || {}),
            disable: this.item.attrs.disable || this.item.attrs.readonly,
            color: (!this.item.feedback && this.value) || this.hasFeedback(this.feedbackValues) ? this.item.attrs.trueColor : this.item.attrs.falseColor,
            fullWidth: true,
            loading: this.isProcessing,
            onClick: () => {
              if (this.item.attrs.pulse) {
                this.emitValuePulse()
              } else {
                this.emitValue(!this.value)
              }
            }
          },
        },
      ]
    },
  }
})