// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
	  'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-post-outline',
	
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    filter: [{ hasChangelog: true }],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                    caption: 'keepEmptyForRootObject',
                  },
                  {
                    type: 'select',
                    default: 'md',
                    label: 'size',
                    scope: 'attrs.size',
                    col: 'col-12',
                    options: [
                      { label: 'xs', value: 'xs' },
                      { label: 'sm', value: 'sm' },
                      { label: 'md', value: 'md' },
                      { label: 'lg', value: 'lg' },
                    ]
                  },
                  {
                    type: 'select',
                    default: 'sm',
                    label: 'padding',
                    scope: 'attrs.padding',
                    col: 'col-12',
                    options: [
                      { label: 'xs', value: 'xs' },
                      { label: 'sm', value: 'sm' },
                      { label: 'md', value: 'md' },
                      { label: 'lg', value: 'lg' },
                    ]
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        changelog: [],
        url: '',
      }
    },

    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    }, 
    // Listen on mounted event...
    mounted() {
      // logic
      this.url = this.panel.getEndpoint(this.computedObject.getProcessUuid())
      this.computedObject.getPathes().forEach((path) => {
        this.changelog.push({
          group: '',
          filter: {
            path,
          }
        })
      })
      this.panel.setPermissions(this.item.rights, this.item)
      this.panel.bus.on('endpoints:fetched', () => {
        if (!this.url) {
          this.url = this.panel.getEndpoint(this.computedObject.getProcessUuid())
        }
      })
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
    },
	
    methods: {
    },
	
    // Define elements within the ui configuration...
    nodes() {
      if (!this.url) {
        return [
          {
            type: 'NgSpinner',
            props: {
              label: 'loading',
            },
          }
        ]
      }
      return [
        // any ui component node can be used...
        {
          type: 'NgDMSChangelogButton',
          props: {
            ...this.item.attrs,
            workspaceUuid: this.design.ctx.workspaceUuid,
			serviceUuids: [...(this.design.ctx.serviceUuids || []), this.computedObject.getProcessManagerUuid()],
            changelog: this.changelog,
            url: this.url,
            fullWidth: true,
            allGroups: true,
          },
        },
      ]
    },
  }
})