// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    excludeComputedAttrs: [
    ],

    icon: 'mdi-numeric',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    'call.disable': 'return ctx.data.rights && ctx.data.rights.read && ctx.data.rights.read.length',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: true,
                    label: 'readonly',
                    scope: 'attrs.readonly',
                    col: 'col-12',
                    'call.disable': 'return ctx.data.rights && ctx.data.rights.write && ctx.data.rights.write.length',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    disableTypes: ['none', 'string', 'bool'],
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'sideSuffix',
                    scope: 'sideSuffix',
                    col: 'col-12',
                    'call.disable': 'return !!ctx.data.attrs.sideLabel'
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'sideLabel',
                    scope: 'attrs.sideLabel',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'hint',
                    scope: 'attrs.hint',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'number',
                    default: 0,
                    label: 'digits',
                    scope: 'attrs.digits',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'number',
                    label: 'min',
                    scope: 'attrs.min',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'number',
                    label: 'max',
                    scope: 'attrs.max',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'color-palette',
                    default: '',
                    label: 'color',
                    scope: 'attrs.bgColor',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        isSending: false,
        value: 0,
        type: 'number',
        computedSideObjects: [],
        sideValue: '',
        hasError: false,
        rights: {
          hidden: false,
          readonly: false,
        }
      }
    },

    created() {
      this.item.valueType = this.type
    },

    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
      
      this.sideLinks = []
      if (this.item.sideSuffix && !this.item.attrs.sideLabel) {
        const pathes = this.computedObject.getPathes(this.item.sideSuffix) || []
        pathes.forEach((path) => {
          const link = this.computedObject.getLink(path)
          const computedOject = this.panel.createComputedObject(link)
          this.panel.on(computedOject, link.path, this.onSideEvent)
          this.computedSideObjects.push(computedOject)
        })
      }
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '',  this.onEvent)
      this.computedSideObjects.forEach((computedObject) => {
        this.panel.off(computedObject, '', this.onSideEvent)
        this.panel.removeComputedObject(computedObject)
      })
    },
	
    methods: {
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        this.value = this.computedObject.formatValue({ value });
      },
      onSideEvent(ev) {
        this.sideValue = this.panel.getEventValue(ev)
      },
      emitValue(value) {
        this.isSending = true
        setTimeout(() => {
          this.isSending = false;
        }, 500)
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value: Number(value),
            }
          ]
        });
      },

      checkMinMax(val) {
          if (this.item.attrs.min !== undefined) {
            if (val < this.item.attrs.min) {
              this.hasError = true
              return this.$t('ruleMinValue', { value: this.item.attrs.min})
            }
          }

          if (this.item.attrs.max !== undefined) {
            if (val > this.item.attrs.max) {
              this.hasError = true
              return this.$t('ruleMaxValue', { value: this.item.attrs.max})
            }
          }
        this.hasError = false
        return true
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgFormInput',
          props: {
            ...this.item.attrs,
            digits: this.computedObject.getDigits(),
			sideLabel: this.item.attrs.sideLabel || this.sideValue,
            modelValue: this.value,
            rules: [
              (val) => this.checkMinMax(val),
            ],
            reactiveRules: true,
            lazyRules: false,
            disableAutofix: true,
            'onUpdate:modelValue': (val) => {		1		
				this.value = val
            },
			'onBlur': () => {
              if (!this.isSending && !this.hasError) {
				this.emitValue(this.value)
              }
			},
            'onKeyup': (ev) => {
              if (ev.altKey || ev.ctrlKey || ev.shiftKey || this.isSending)  {
                return
              }

              if (ev.key === 'Enter' && !this.hasError) {
				this.emitValue(this.value)
              }
            },
            type: this.type,
            step: this.computedObject.getDigits()? Number(`0.${'1'.padStart(this.computedObject.getDigits(), '0')}`) : 1,
          },
        },
      ]
    },
  }
})