// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-form-select',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'readonly',
                    scope: 'attrs.readonly',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'hint',
                    scope: 'attrs.hint',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'component',
                    element: 'NgSeparator',
                    scope: '',
                    attrs: {
                      inset: false,
                    },
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'dynamicOptions',
                    hint: 'dynamicOptionsDescription',
                    scope: 'attrs.dynamicOptions',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'key-value',
                    scope: 'attrs.options',
                    label: 'options',
                    hint: 'widgetSelectOptionsDescription',
                    keyIdent: 'label',
                    default: [],
                    schema: [
                      { name: 'label', i18nLabel: 'label', field: 'label', valueType: 'text' },
                      { name: 'value', i18nLabel: 'value', field: 'value'  },
                    ],
                    col: 'col-12',
                    'call.hidden': '!!ctx.data.attrs.dynamicOptions'
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'labelSuffix',
                    hint: 'regExLabelDMS',
                    scope: 'attrs.dynamicOptionsLabelSuffix',
                    col: 'col-12 col-md-6',
                    'call.hidden': '!ctx.data.attrs.dynamicOptions',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'valueSuffix',
                    hint: 'regExValueDMS',
                    scope: 'attrs.dynamicOptionsValueSuffix',
                    col: 'col-12 col-md-6',
                    'call.hidden': '!ctx.data.attrs.dynamicOptions',
                    sideSlot: 'link',
                  },
                  {
                    type: 'number',
                    default: undefined,
                    label: 'optionsAmount',
                    scope: 'attrs.dynamicOptionAmount',
                    col: 'col-12 col-md-6',
                    'call.hidden': '!ctx.data.attrs.dynamicOptions',
                    sideSlot: 'link',
                  },
                  {
                    type: 'component',
                    element: 'NgSeparator',
                    scope: '',
                    attrs: {
                      inset: false,
                    },
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        value: 0,
        type: 'select',
        options: [],
        dynamicOptions: {
          labels: [],
          values: [],
        },
        dynamicOptionsTimeout: 0,
        subscribedPathes: [],
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
          if (this.item.attrs.options === 'string') {
            this.options = this.getStaticOptions()
          }
        },
        deep: true,
      },
    },

    computed: {
      filteredOptions () {
        return this.handleDynamicOptionsAmount()
      }
    },
      
    created() {
      this.item.valueType = this.type
    },

    // Listen on mounted event...
    mounted() {
      // logic
      this.options = this.item.attrs.dynamicOptions ? [] : this.getStaticOptions()
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
      this.fetchDynamicOptions()
    },
	
    // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '',  this.onEvent)
      this.unfetchDynamicOptions()
      this.unfetchReferencedOptions()
    },
	
    methods: {
      handleDynamicOptionsAmount() {
        if (this.item.attrs.dynamicOptions && this.item.attrs.dynamicOptionAmount) {
          return this.options.slice(0, this.item.attrs.dynamicOptionAmount <= this.options.length ? this.item.attrs.dynamicOptionAmount : this.options.length)
        }

        return this.options && this.options.length ? this.options : (this.item.attrs.options || [])
      },
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        this.value = value;
      },
      emitValue(value) {
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value,
            }
          ]
        });
      },
      getStaticOptions() {
        let options = []
        let referencedSuffixes = []
        if (typeof this.item.attrs.options === 'string') {
          try {
            options = JSON.parse(this.item.attrs.options || '[]')
          } catch (e) {
            console.error(e)
            options = []
          }
        } else {
          options = JSON.parse(JSON.stringify(this.item.attrs.options || []))
          options.forEach((option) => {
            let hasSuffix = false
            if (String(option.label).startsWith('@')) {
              hasSuffix = true
              referencedSuffixes.push(option.label)
            }

            if (String(option.value).startsWith('@')) {
              hasSuffix = true
              referencedSuffixes.push(option.value)
            }

            if (hasSuffix) {
              option.badge = {
                value: this.ctx.i18n.i18n.t('loading'),
                color: 'warning',
              }
            }
          })
        }

        if (referencedSuffixes.length) {
          this.fetchReferencedOptions(referencedSuffixes)
        }

        return JSON.parse(JSON.stringify(options))
      },
      createReferencedOptions(ev) {
        const value = this.panel.getEventValue(ev)
        clearTimeout(this.dynamicOptionsTimeout)
        
        this.item.attrs.options.forEach((option, idx) => {
          if (ev.payload.tag.cmd === option.label) {
            this.options[idx].label = value
          } else if (ev.payload.tag.cmd === option.value) {
            this.options[idx].value = value
          }
          if (!String(this.options[idx].label).startsWith('@')) {
            this.options[idx].badge = undefined;
          }
        })

        // debounce a little bit before rerender select element
        this.dynamicOptionsTimeout = setTimeout(() => {
          this.options = JSON.parse(JSON.stringify(this.options || []))
        }, 100)
      },
      onEventReferencedOptions(ev) {
        this.createReferencedOptions(ev)
      },
      unfetchReferencedOptions() {
        const topic = this.computedObject.getIdent()
        this.panel.off(this.computedObject.getProcessUuid(), topic, this.onEventReferencedOptions)
      },
      fetchReferencedOptions(pathes) {
        const topic = this.computedObject.getIdent()
  
        this.panel.on(this.computedObject.getProcessUuid(), topic, this.onEventReferencedOptions)

        const subscribe = pathes.map((path) => ({
          // remove @ char at the begining
          path: this.computedObject.getPath(path.substring(1)),
          event: ['onChange'],
          tag: {
            topic,
            cmd: path,
          },
        }))
        
        this.panel.emit(this.computedObject.getProcessUuid(), {
          subscribe,
        }, undefined, true);
      },
 
      createDynamicOptions(ev) {
        const cmd = ev.payload.tag.cmd
        const value = this.panel.getEventValue(ev)
        const path = this.panel.getEventPath(ev)

        if (cmd === 'label') {
          const item = this.dynamicOptions.labels.find((i) => i.path === path)
          if (item) {
            item.value = value
          } else {
            this.dynamicOptions.labels.push({ value, path })
          }
        } else if (cmd === 'value') {
          const item = this.dynamicOptions.values.find((i) => i.path === path)
          if (item) {
            item.value = value
          } else {
            this.dynamicOptions.values.push({ value, path })
          }
        }

        clearTimeout(this.dynamicOptionsTimeout)

        this.dynamicOptionsTimeout = setTimeout(() => {
          this.options = []
          const options = []
          this.dynamicOptions.labels.forEach((l, idx) => {
            if (this.dynamicOptions.values.length === this.dynamicOptions.labels.length) {
              const value = this.dynamicOptions.values[idx].value
              const valuePath = this.dynamicOptions.values[idx].path
              const label = l.value
              const labelPath = l.path
              const option =  { label, value, opts: { labelPath, valuePath } }
              options.push(option)
            }
          })
          this.options = options.filter((option) => !!option.label)
          this.subscribeDynamicOptions()
        }, 50)
      },
      onEventDynamicOptions(ev) {
        this.createDynamicOptions(ev)
      },
      unfetchDynamicOptions() {
        const topic = this.computedObject.getIdent()
        this.subscribedPathes = []
        this.panel.off(this.computedObject.getProcessUuid(), topic, this.onEventDynamicOptions)
        this.panel.off(this.computedObject.getProcessUuid(), `sub-${topic}`, this.onEventDynamicOptions)
      },
      fetchDynamicOptions() {
        if (this.item.attrs.dynamicOptions && this.item.attrs.dynamicOptionsLabelSuffix && this.item.attrs.dynamicOptionsValueSuffix) {
          this.dynamicOptions = {
            labels: [],
            values: [],
          }

          this.unfetchDynamicOptions();
          this.subscribedPathes = []
          const labelSuffix = `.*:${this.item.attrs.dynamicOptionsLabelSuffix}`
          const valueSuffix = `.*:${this.item.attrs.dynamicOptionsValueSuffix}`
          const path = this.computedObject.getParent().getPath()
          const topic = this.computedObject.getIdent()

          this.panel.on(this.computedObject.getProcessUuid(), topic, this.onEventDynamicOptions)

          this.panel.emit(this.computedObject.getProcessUuid(), {
            get: [
              {
                path,
                query: {
                  regExPath: labelSuffix,
                  maxDepth: 0,
                },
                tag: {
                  topic,
                  cmd: 'label',
                },
              },
              {
                path,
                query: {
                  regExPath: valueSuffix,
                  maxDepth: 0,
                },
                tag: {
                  topic,
                  cmd: 'value',
                },
              }
            ],
          }, undefined, true);
        }
      },
      subscribeDynamicOptions() {
        const topic = `sub-${this.computedObject.getIdent()}`

        let subscribe = [];

        this.options.forEach((option) => {
          subscribe.push({ path: option.opts.labelPath, event: ['onChange'], tag: { topic, cmd: 'label', } })
          subscribe.push({ path: option.opts.valuePath, event: ['onChange'], tag: { topic, cmd: 'value', } })
        })

        // filter out already subscribed pathes
        subscribe = subscribe.filter((sub) => !this.subscribedPathes.includes(sub.path))

        if (subscribe.length <= 0) {
          return
        }

        this.panel.off(this.computedObject.getProcessUuid(), topic, this.onEventDynamicOptions)
        this.panel.on(this.computedObject.getProcessUuid(), topic, this.onEventDynamicOptions)
        this.panel.emit(this.computedObject.getProcessUuid(), { subscribe }, undefined, true)
        this.subscribedPathes = subscribe.map((sub) => sub.path)
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgFormInput',
          props: {
            ...this.item.attrs,
            modelValue: this.value,
            'onUpdate:modelValue': (val) => {
              this.value = val
              this.emitValue(val)
            },
            type: this.type,
            options: this.filteredOptions,
          },
        },
      ]
    },
  }
})
