// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],
    
    icon: 'mdi-format-text-variant',

    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'readonly',
                    scope: 'attrs.readonly',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    disableLinkTypes: ['none'],
                    defaultType: 'template',
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    enableTypes: ['string'],
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'sideSuffix',
                    scope: 'sideSuffix',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'hint',
                    scope: 'attrs.hint',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        isSending: false,
        value: 0,
        label: this.item.attrs.label || 'value',
        type: 'text',
        digits: undefined,
        sideLinks: [],
        sideValue: '',
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },
    created() {
      this.item.valueType = this.type
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.digits = this.computedObject.getDigits()
      this.panel.setPermissions(this.item.rights, this.item)

      this.sideLinks = []

      if (this.item.sideSuffix) {
        const processUuid = this.computedObject.getProcessUuid()
        const pathes = this.computedObject.getPathes(this.item.sideSuffix) || []
        pathes.forEach((path) => {
          const link = this.computedObject.getLink(path)
          this.panel.setLink(link)
          this.panel.on(processUuid, link.path, this.onSideEvent)
          this.sideLinks.push(link)
        })
      }
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.item, this.onEvent)
    },
	
    methods: {
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        this.value = this.computedObject.formatValue({ value });
      },
      emitValue(value) {
        this.isSending = true
        setTimeout(() => {
          this.isSending = false;
        }, 500)
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value: value,
            }
          ]
        });
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgFormInput',
          props: {
            ...this.item.attrs,
            modelValue: this.value,
			sideLabel: this.item.attrs.sideLabel || this.sideValue,
            'onUpdate:modelValue': (val) => {
              this.value = val
            },
            'onBlur': () => {
              if (!this.isSending) {
				this.emitValue(this.value)
              }
			},
            'onKeyup': (ev) => {
              if (ev.altKey || ev.ctrlKey || ev.shiftKey || this.isSending)  {
                return
              }

              if (ev.key === 'Enter') {
				this.emitValue(this.value)
              }
            },
            type: this.type,
          },
        },
      ]
    },
  }
})