// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-clock-time-eight-outline',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'readonly',
                    scope: 'attrs.readonly',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'hint',
                    scope: 'attrs.hint',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'select',
                    default: 'HH:mm',
                    label: 'format',
                    scope: 'attrs.format',
                    hint: 'dataFormat',
                    col: 'col-12 col-md-6',
                    options: [
                      { label: 'auto', value: '' },
                      { label: 'HH:mm', value: 'HH:mm' },
                      { label: 'HH:mm:ss', value: 'HH:mm:ss' },
                      { label: 'seconds', value: 's' },
                      { label: 'milliseconds', value: 'ms' },
                    ]
                  },
                  {
                    type: 'select',
                    default: 'HH:mm',
                    label: 'mask',
                    scope: 'attrs.mask',
                    hint: 'displayMask',
                    col: 'col-12 col-md-6',
                    options: [
                      { label: 'HH:mm', value: 'HH:mm' },
                      { label: 'HH:mm:ss', value: 'HH:mm:ss' },
                    ]
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        value: '',
        type: 'time',
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },
    created() {
      this.item.valueType = this.type
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '',  this.onEvent)
    },
    
    methods: {
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        if (value !== undefined) {
          this.value = value 
        }
      },
      
      emitValue(value) {
        const path = this.computedObject.getPath();
        this.panel.emit(this.computedObject, {
          set: [
            {
              path,
              value,
            }
          ]
        });
      }
    },
    
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgFormInput',
          props: {
            ...this.item.attrs,
            modelValue: this.value,
            'onUpdate:modelValue': (val) => {
              this.value = val
            },
			'onBlur': () => {
              this.emitValue(this.value)
			},
            type: this.type,
          },
        },
      ]
    },
  }
})