// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
	  'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-toggle-switch',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'readonly',
                    scope: 'attrs.readonly',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'hint',
                    scope: 'attrs.hint',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'select',
                    default: 'boolean',
                    label: 'type',
                    scope: 'attrs.valueType',
                    options: [
                      { label: 'string', value: 'string' },
                      { label: 'number', value: 'number' },
                      { label: 'boolean', value: 'boolean' },
                    ],
                    col: 'col-12',
                  },
                  {
                    type: 'number',
                    default: undefined,
                    label: 'bit',
                    scope: 'attrs.bit',
                    col: 'col-12',
                    'call.hidden': `return ctx.data.attrs.valueType !== 'boolean'`,
                    min: 0,
                    max: Number.MAX_SAFE_INTEGER,
                    sideSlot: 'link',
                  },
                  {
                    'call.type': `
                      const valueType = ctx.data.attrs.valueType
                      let type = 'boolean'
                      switch (valueType) {
                        case 'string':
                          type = 'text'
                          break
                        case 'boolean':
                          type = 'checkbox'
                          break
                        case 'number':
                          type = 'number'
                          break
                      }
                      return type
                    `,
                    default: true,
                    label: 'trueValue',
                    scope: 'attrs.trueValue',
                    col: 'col-12 col-md-6',
                    sideSlot: 'link',
                  },
                  {
                    'call.type': `
                      const valueType = ctx.data.attrs.valueType
                      let type = 'boolean'
                      switch (valueType) {
                        case 'string':
                          type = 'text'
                          break
                        case 'boolean':
                          type = 'checkbox'
                          break
                        case 'number':
                          type = 'number'
                          break
                      }
                      return type
                    `,
                    default: false,
                    label: 'falseValue',
                    scope: 'attrs.falseValue',
                    col: 'col-12 col-md-6',
                    sideSlot: 'link',
                  },
                  {
                    type: 'color-palette',
                    default: '',
                    label: 'color',
                    scope: 'attrs.color',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        value: false,
        type: 'toggle',
		originalValue: false,
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    }, 
    created() {
      this.item.valueType = 'checkbox'
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '',  this.onEvent)
    },
	
    methods: {
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        this.originalValue = value;
        if (this.item.attrs.bit !== undefined && this.item.attrs.bit === Number(this.item.attrs.bit)) {
          this.value = !!(value & 1 << this.item.attrs.bit)
        } else {
          this.value = value;
        }
      },
      emitValue(value) {
        const path = this.computedObject.getPath();
        if (this.item.attrs.bit !== undefined && this.item.attrs.bit === Number(this.item.attrs.bit)) {
          if (value) {
            this.originalValue |= 1 << this.item.attrs.bit 
          } else {
            this.originalValue &= ~(1 << this.item.attrs.bit)
          }
          this.panel.emit(this.computedObject, {
            set: [
              {
                path,
                value: this.originalValue,
              }
            ]
          });
        } else {
          this.panel.emit(this.computedObject, {
            set: [
              {
                path,
                value,
              }
            ]
          });
        }
      },
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgFormInput',
          props: {
            ...this.item.attrs,
            modelValue: this.value,
            'onUpdate:modelValue': (val) => {
              this.value = val
              this.emitValue(val)
            },
            type: this.type,
          },
        },
      ]
    },
  }
})