// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'markdown',
      'design',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-card-text-outline',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'flat',
                    scope: 'attrs.flat',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'template',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'title',
                    scope: 'attrs.title',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'select',
                    default: '',
                    label: 'type',
                    scope: 'valueType',
                    col: 'col-12',
                    options: [
                      { label: '-', value: '' },
                      { label: 'number', value: 'number'},
                      { label: 'boolean', value: 'boolean'},
                      { label: 'string', value: 'string'},
                    ]
                  },
                  {
                    type: 'editor',
                    default: '',
                    label: 'markdown',
                    scope: 'attrs.markdown',
                    col: 'col-12',
                    lang: 'markdown',
                    hint: 'Placeholder: {SUFFIX}, Format: {SUFFIX|%0.2f}'
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },	
	
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        originalMarkdown: this.item.attrs.markdown || '',
        markdownValue: this.item.attrs.markdown || '',
        payloads: {},
        digits: {},
        markdownKeyMap: {},
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    }, 

    created() {
      if (!this.item.attrs.markdown) {
        this.item.attrs.markdown = ''
      }
      this.item.disableDeep = true;
      if (!this.item.components) {
        this.item.components = []
      }
      this.item.valueType = 'text'
    },
    // Listen on mounted event...
    mounted() {
      // logic
      this.item.valueType = 'text'
      if (this.item && this.item.link && this.item.link.path) {
        this.panel.setPermissions(this.item.rights, this.item)

        // Try to get digits from link
        const suffixes = this.computedObject.getPathSuffixes()
        const template = this.computedObject.getParent().getComputedObject()
        suffixes.forEach((suffix) => {
          if (template && template.customData && template.customData.number && template.customData.number[suffix]) {
            this.digits[suffix] = template.customData.number[suffix]
          }
        })
      }

      const markdownKeys = this.item.attrs.markdown.match(/\{(.+?)\}/gi) || []
      markdownKeys.forEach((key) => {
        const parts = key.replace(/(\{|\})/gi, '').split('|');
        const suffix = parts[0];
        const format = parts.length >= 2 ? parts[1] : '';
        if (!this.markdownKeyMap[suffix]) {
          this.markdownKeyMap[suffix] = []
        }
        this.markdownKeyMap[suffix].push({
          key,
          suffix,
          format,
        });
      })

      this.panel.on(this.computedObject, '', this.onEvent)
      this.renderMarkdown()
    },
	
    // Listen on mounted event...
    unmounted() {
      // logic
      this.panel.off(this.computedObject, '', this.onEvent)

    },
	
    methods: {
      onEvent(ev) {
        const path = this.panel.getEventPath(ev)
        const value = this.panel.getEventValue(ev)
        this.payloads[path] = value
        this.renderMarkdown()
      },
      getRenderValue(suffix, value) {
        let result = ''
        switch (typeof value) {
          case 'number':
            if (this.digits[suffix] !== undefined) {
              result = value.toFixed(this.digits[suffix])
            } else {
              result = value
            }
            break;
          case 'string':
            result = this.ctx.i18n.t(value)
            break;
          case 'boolean':
            result = this.ctx.i18n.t(value ? 'on' : 'off')
            break;
          default:
            result = value
            break;
        }
        return result
      },
      renderMarkdown() {
        const suffixes = this.computedObject.getPathSuffixes()
        let markdown = this.originalMarkdown
        Object.keys(this.payloads).forEach((path) => {
          let value = this.payloads[path]
          const suffix = suffixes.find((s) => path.endsWith(s))
          const markdownItems = this.markdownKeyMap[suffix];
          if (markdownItems) {
            markdownItems.forEach((markdownItem) => {
              if (markdownItem.format) {
                markdown = markdown.replaceAll(markdownItem.key, this.panel.sprintf(markdownItem.format, value))
              } else {
                markdown = markdown.replaceAll(markdownItem.key, this.getRenderValue(markdownItem.suffix, value))
              }
            })
          }
        })
        this.markdownValue = markdown
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
                {
          type: 'NgCard',
          props: {
            ...( this.item.attrs || {}),
            class: 'full-height',
          },
          children: {
            default: () => {
              return [
                // child nodes or string content
                this.item.attrs.markdown ? {
                  type: 'NgCardSection',
                  props: {

                  },
                  children: {
                    default: () => {
                      return [
                        // child nodes or string content
                        {
                          type: 'NgMarkdown',
                          props: {
                            ...this.item.attrs,
                            markdown: this.markdownValue,
                          },
                        },
                      ]
                    }
                  },
                } : '',
                {
                  type: 'NgCardSection',
                  props: {
                  },
                  slots: { 
                    default: () => {
                      return [
                        // child nodes or string content
                        this.slots.nested ? this.slots.nested(this.item) : ''
                      ]
                    }
                  }
                },
              ]
            }
          },
        },
      ]
    },
  }
})