// Copyright 2021 MST Systemtechnik AG. All Rights Reserved.

window.addEventListener('load', function () {

    'use strict';

    // Error panel
    let modalTimer = 0;
    function showError(title, text, displayDuration) {
        let infoPanelEl = document.getElementById('error-panel');
        infoPanelEl.innerHTML = '<b>' + title + '</b><br>';
        infoPanelEl.innerHTML += text;
        infoPanelEl.className = 'sticky';
        // Start timer for auto close
        if (modalTimer) {
            clearTimeout(modalTimer);
        }
        if (displayDuration) {
            modalTimer = setTimeout(function () {
                infoPanelEl.className = 'sticky invisible';
            }, displayDuration);
        }
    }
    
    // Job request handling
    let jobRequest = null;
    let jobTimer = 0;

    function getXmlHttpjobRequestObject() {
        if (!jobRequest) {
            // Create a new XMLHttpRequest object
            jobRequest = new XMLHttpRequest();
        }
        return jobRequest;
    }
    
    function pollInstancesJobs() {
        if (jobTimer) {
            clearTimeout(jobTimer);
            jobTimer = 0;
        }
        let now = new Date();
        jobRequest = getXmlHttpjobRequestObject();
        let url = '/v1/manager/jobs?_' + now.getTime();
        jobRequest.onreadystatechange = function () {
            // Check response is ready or not
            if (jobRequest.readyState === 4) {
                if (jobRequest.status === 200) {
                    // Create / update status list
                    createJobList(JSON.parse(jobRequest.responseText));
                    // Update the live data every 1.5 seconds
                    jobTimer = setTimeout(pollInstancesJobs, 1500);
                } else {
                    let statusText = jobRequest.status + ' - ' + jobRequest.statusText;
                    if (jobRequest.status === 0) {
                        statusText = 'No Connection';
                    }
                    showError('Status Poll', 'Error ' + statusText + '<br><br>' + jobRequest.responseText, 6000);
                    // Retry in 5 sec
                    jobTimer = setTimeout(pollInstancesJobs, 5000);
                }
            }
        };
        // asynchronous requests
        jobRequest.open("GET", url, true);
        jobRequest.timeout = 5000;
        // Send the request over the network
        jobRequest.send();
    }

    // Start first poll
    pollInstancesJobs();

    function createSpanElementWithText(text, className) {
        let element = document.createElement('span');
        element.innerHTML = text;
        if (className !== undefined) {
            element.className = className;
        }
        return element;
    }

    function createJobList(jobStatus) {

        let jobInfoEl = document.getElementById('job-info');
        jobInfoEl.innerHTML = 
            'Total Start: <b>' + jobStatus.totalStart + '</b>, ' + 
            'Stop: <b>' + jobStatus.totalStop + '</b>, ' + 
            'Cron: <b>' + jobStatus.totalCron + '</b>, ' + 
            'Monitor: <b>' + jobStatus.totalMonitor + '</b>' +
            '<br>ProcessControllers: <b>' + jobStatus.totalProcessControllers + '</b>, ' +
            'DmsHandlers: <b>' + jobStatus.totalDmsHandlers + '</b>, ' +
            'HttpConnections: <b>' + jobStatus.totalHttpConnections + '</b>';

        // Iterate all types
        ['start', 'stop', 'cron', 'monitor'].forEach(function (type) {
            let typeEl = document.getElementById('job-type-' + type);
            typeEl.innerHTML = '';

            if (jobStatus[type].length > 0) {
                // Add table
                let jobTbl = document.createElement('table');
                let jobTblHdr = document.createElement('thead');
                let jobTableHdrStr = ['Instance', 'Status', 'Process', 'ProcessStatus'];
                jobTableHdrStr.forEach(function (title) {
                    jobTblHdr.appendChild(document.createElement("th")).appendChild(document.createTextNode(title));
                });
                jobTbl.appendChild(jobTblHdr);
                
                // Iterate all instances
                jobStatus[type].forEach(function(instanceObj) {

                    // Append row
                    let jobTblRow = jobTbl.insertRow();
                    // Add instance cells
                    let jobTblCell = jobTblRow.insertCell()
                    jobTblCell.setAttribute('rowspan', instanceObj.processes.length);
                    jobTblCell.appendChild(createSpanElementWithText(instanceObj.instance));
                    jobTblCell = jobTblRow.insertCell()
                    jobTblCell.setAttribute('rowspan', instanceObj.processes.length);
                    jobTblCell.appendChild(createSpanElementWithText(instanceObj.status, 'status-' + instanceObj.status));

                    // Add process cells
                    instanceObj.processes.forEach(function(processObj, idx) {
                        if (idx !== 0)
                        {
                            jobTblRow = jobTbl.insertRow();
                        }
                        jobTblRow.insertCell().appendChild(createSpanElementWithText(processObj.name + ' (' + processObj.description + ')'));
                        let processStatusText = processObj.status;
                        if (processObj.status === 'WaitDelay' || processObj.status === 'WaitCron') {
                            processStatusText += ' - ' + new Date(processObj.tsNextStart).toLocaleString();
                        }
                        else if (processObj.status === 'WaitOrder') {
                            processStatusText += ' (' + processObj.startOrder + ')';
                        }
                        if (processObj.status === 'WaitRunControl' || type === 'monitor') {
                            processStatusText += ' (' + processObj.runMonControlMethod + ' / "' + processObj.runMonControlArgument + '")';
                        }
                        if (processObj.processStatus !== '') {
                            processStatusText += ' [' + processObj.processStatus + ']';
                        }
                        jobTblRow.insertCell().appendChild(createSpanElementWithText(processStatusText, 'status-' + processObj.status));
                    });
                });

                // Append table to formular
                typeEl.appendChild(jobTbl);
            }
            else {
                typeEl.innerHTML = "No jobs";
            }
        });
    }

});