// Copyright 2022 MST Systemtechnik AG. All Rights Reserved.

window.addEventListener('load', function () {

    'use strict';

    // Error panel
    let modalTimer = 0;
    function showError(title, text, displayDuration) {
        let infoPanelEl = document.getElementById('error-panel');
        infoPanelEl.innerHTML = '<b>' + title + '</b><br>';
        infoPanelEl.innerHTML += text;
        infoPanelEl.className = 'sticky';
        // Start timer for auto close
        if (modalTimer) {
            clearTimeout(modalTimer);
        }
        if (displayDuration) {
            modalTimer = setTimeout(function () {
                infoPanelEl.className = 'sticky invisible';
            }, displayDuration);
        }
    }

    let urlParams = new URLSearchParams(window.location.search);
    let processUuid = urlParams.get('processUuid');
    let serverPort = urlParams.get('serverPort');
    let currentStatusId = urlParams.get('statusId');
	let wsProtocol = 'ws:';
	if (location.protocol === 'https:') {
		wsProtocol = 'wss:';
	}
	let socket = new WebSocket(wsProtocol + '//' + location.hostname + ':' + serverPort + 
      location.pathname.substring(0, location.pathname.lastIndexOf('/')) + '/v1/status');

    let sortedHeader = '';
    let sortedAsc = true;

    let maxLines = 100;
    document.getElementById('max-line-select').onchange = function(event) {
        maxLines = parseInt(event.target.value);
        // Reload status
        socket.send(JSON.stringify({ command: 'subscribe', statusId: currentStatusId }));
    };    

    window.addEventListener("pagehide", function () {
        socket.onclose = function () {};
        if (socket.readyState == WebSocket.OPEN)
        {
            socket.send(JSON.stringify({ command: 'unsubscribe', statusId: currentStatusId }));
            socket.close();
        }
    });

    socket.onopen = function () {
        // Init first status
        socket.send(JSON.stringify({ command: 'subscribe', statusId: currentStatusId }));
    };

    function htmlEncode(str) {
        return str.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;").replace(/\n/g, "<br>");
    }

    let asc = true;
    let isStatic = true;
    let columnsSize = 0;
    function handleInitEvent(event) {
        // Set status title
        document.getElementsByTagName('h1')[0].innerHTML = event.statusTitle;

        // Add table
        let statusContainer = document.getElementById('status-container');
        statusContainer.innerHTML = '';
        let statusTbl = document.createElement('table');
        statusTbl.id = 'status-table';
        let statusTblHdr = document.createElement('thead');
        event.columnHeaders.forEach(function (header) {
            statusTblHdr.appendChild(document.createElement("th")).appendChild(document.createTextNode(header.title));
        });
        columnsSize = event.columnHeaders.length;
        statusTbl.appendChild(statusTblHdr);
        let statusBodyEl = document.createElement('tbody');
        statusBodyEl.id = 'status-table-rows';
        statusTbl.appendChild(statusBodyEl);
        statusContainer.appendChild(statusTbl);

        // Add sortable
        const getCellValue = (tr, idx) => tr.children[idx].innerText || tr.children[idx].textContent;

        const comparer = (idx, asc) => (a, b) => ((v1, v2) => 
            v1 !== '' && v2 !== '' && !isNaN(v1) && !isNaN(v2) ? v1 - v2 : v1.toString().localeCompare(v2)
            )(getCellValue(asc ? a : b, idx), getCellValue(asc ? b : a, idx));
        
        document.querySelectorAll('th').forEach(th => th.addEventListener('click', (() => {
            const table = th.closest('table');
            document.querySelectorAll('.th-sorted').forEach(x => x.classList.remove('th-sorted'));
            th.classList.add('th-sorted');
            Array.from(table.querySelectorAll('tr:nth-child(n+1)'))
                .sort(comparer(Array.from(th.parentNode.children).indexOf(th), asc = !asc))
                .forEach(tr => table.appendChild(tr) );
        })));

        // Add event list options
        isStatic = event.isStatic;
        if (!isStatic) {
            document.getElementById('list-options').className = '';
            statusTbl.style.setProperty('width', '0%');
        }
        else {
            // statusTbl.classList.add('static-table');
        }
    }

    function openSubState(statusId) {
        window.open('/status.html?processUuid=' + processUuid + '&serverPort=' + serverPort + '&statusId=' + statusId, '_blank').focus();
    }

    function doAction(actionId) {
        socket.send(JSON.stringify({ command: 'action', statusId: currentStatusId, actionId: actionId }));
    }

    function handleRowsEvent(event) {
        let statusContentEl = document.getElementById('status-table-rows');

        // Add / update rows to table
        let textFilter = document.getElementById('text-filter').value;
        event.rows.forEach(function (row) {
            // Check existing row
            let statusTblRow = undefined;
            let newRow = false;
            if (row.rowId === '') {
                // Row to insert
                if (textFilter !== '') {
                    // Check filter
                    let filtered = true;
                    row.cells.forEach(function (cell) {
                        if (cell.format === 'string') {
                            if (cell.content.includes(textFilter)) {
                                filtered = false;
                                return;
                            }
                        }
                    });
                    if (filtered) {
                        return;
                    }
                }
                statusTblRow = statusContentEl.insertRow(0);
                newRow = true;
            }
            else {
                // Check indexed row
                statusTblRow = document.getElementById('status-table-row-' + row.rowId);
                if (!statusTblRow) {
                    statusTblRow = statusContentEl.insertRow(-1);
                    statusTblRow.id = 'status-table-row-' + row.rowId;
                    newRow = true;
                }
            }
            if (newRow) {
                // Add all cells (depending on header)
                for (let i=0; i<columnsSize; i++) {
                    statusTblRow.insertCell();
                }
            }

            // Update cells
            row.cells.forEach(function (cell) {
                let cellEl = statusTblRow.cells[cell.columnIndex];
                let newValue = '';
                cellEl.className = 'status-' + cell.status;
                // Set content, depending on type
                switch (cell.format) {
                    case 'string':
                        newValue = htmlEncode(cell.content);
                        break;
                    case 'timestamp':
						let ts = new Date(cell.content);
						ts.setMinutes(ts.getMinutes() - ts.getTimezoneOffset());
                        newValue = ts.toISOString().replace('T', ' ').replace('Z', '');
                        break;
                    case 'numeric':
                        newValue = cell.content.toLocaleString();
                        cellEl.classList.add('center');
                        break;
                    case 'bool':
                        newValue = '<input type="checkbox"' + (cell.content ? ' checked="checked"' : "") + ' disabled="disabled"></input>';
                        cellEl.classList.add('center');
                        break;
                    case 'subStates':
                    case 'actions':
                        cellEl.innerHTML = '';
                        let selectEl = document.createElement("SELECT");
                        let opt = document.createElement("option");
                        opt.text = 'Select ' + (cell.format === 'subStates' ? 'sub status...' : 'action...');
                        opt.value = '';
                        selectEl.appendChild(opt);
                        cell.options.forEach(function (option) {
                            let optDetail = document.createElement("option");
                            optDetail.text = option.text;
                            optDetail.value = option.id;
                            selectEl.appendChild(optDetail);
                        });
                        selectEl.onchange = (evt) => {
                            if (evt.target.value !== '') {
                                if (cell.format === 'subStates') {
                                    openSubState(evt.target.value);
                                }
                                else {
                                    doAction(evt.target.value);
                                }
                                evt.target.value = '';
                            }
                        }
                        cellEl.appendChild(selectEl);

                        // cellEl.classList.add('center');
                        newValue = cellEl.innerHTML;
                        break;
                    default:
                        newValue = htmlEncode(cell.content);
                        break;
                }
                if (cellEl.innerHTML !== newValue) {
                    if (!isStatic || cellEl.innerHTML !== '') {
                        cellEl.classList.add('animated');
                    }
                    cellEl.innerHTML = newValue;
                    // Restart animation
                    let newone = cellEl.cloneNode(true);
                    cellEl.parentNode.replaceChild(newone, cellEl);
                }
    
            });

            if (!isStatic) {
                while (statusContentEl.rows.length > maxLines) {
                    statusContentEl.deleteRow(-1);
                }
            }
        });
    }

    socket.onmessage = function (data) {
        let response = JSON.parse(data.data);

        if (response.events === undefined) {
            showError('No events received', data, 5000);
        }

        else {
            response.events.forEach(function (event) {

                if (event.event == 'init') {
                    handleInitEvent(event);
                }
        
                else if (event.event == 'rows') {
                    if (document.getElementById('list-enabled').checked) {
                        handleRowsEvent(event);
                    }
                }

                else if (event.event == 'error') {
                    showError('Event error', 'Error received:' + event.message, 5000);
                }
        
                else {
                    showError('Event error', 'Invalid event received:' + event.event, 5000);
                }
            });
        }
    };

    socket.onclose = function (event) {
        if (event.wasClean) {
            showError('WebSocket closed cleanly', 'code=${event.code} reason=${event.reason}', 0);
        } else {
            showError('WebSocket closed', 'Please reload page', 0);
        }
    };

    socket.onerror = function (error) {
        showError('WebSocket error', error.message, 5000);
    };

});